'PhysLite
'TODO: ??
SuperStrict

Rem
bbdoc: Miscellaneous/PhysLite
End Rem
Import BRL.Math
Import BRL.Map
Import BRL.Max2D
Import BRL.EndianStream
Import BRL.PNGLoader
Import BRL.Retro

Include "AdvBody.bmx"
Include "ProcFunc.bmx"

Rem
bbdoc: Physics simulation object
End Rem
Type TPhysics
	Global cgCounter:Int=-2
	
	Field boundsMaxX:Float
	Field boundsMaxY:Float
	Field boundsMinX:Float
	Field boundsMinY:Float
	Field friction:Float=1.0
	Field gravityX:Float
	Field gravityY:Float
	Field i:Int
	
	Field gridon:Int=False
	Field shiftgrid:TList[MAX_GRID,MAX_GRID]
	Field gridsize:Int
	Field ratiox:Float, ratioy:Float
	
	Field drawOffsetX:Float=0.0
	Field drawOffsetY:Float=0.0
	Field drawScale:Float=1.0
	Field centerX:Float=GraphicsWidth()*.5
	Field centerY:Float=GraphicsHeight()*.5
	
	Field drawRot:Float=0
	Field drawIX:Float=Cos(drawRot)*drawScale
	Field drawIY:Float=-Sin(drawRot)*drawScale
	Field drawJX:Float=Sin(drawRot)*drawScale
	Field drawJY:Float=Cos(drawRot)*drawScale
	
	Const MAX_GRID:Int=128
	
	Rem
	bbdoc: Creates a new physics simulation.
	returns: A TPhysics object.
	about: @Note: that the gravity @gravY parameter is before the @gravX parameter.
	End Rem
	Function Create:TPhysics( width:Float, height:Float, gravY:Float, gravX:Float=0.0, i:Int=1 )
		Local p:TPhysics=New TPhysics
		p.boundsMinX=0
		p.boundsMinY=0
		p.boundsMaxX=width
		p.boundsMaxY=height
		p.gravityX=gravX
		p.gravityY=gravY
		p.i=i
		p.friction=1.0
		p.gridsize=8
		p.ratiox=p.gridsize/width
		p.ratioy=p.gridsize/height
		Return p
	End Function
	
	Rem
	bbdoc: Ends the physics simulation.
	End Rem
	Function Free()
		phys.boundsMinX=0
		phys.boundsMinY=0
		phys.boundsMaxX=0
		phys.boundsMaxY=0
		phys.gravityX=0
		phys.gravityY=0
		phys.i=0
		phys.friction=0
		TConstraint.array=Null
		TConstraint.stack=Null
		TMass.array=Null
		TMass.stack=Null
		TZone.array=Null
		TZone.stack=Null
		phys=Null
		
	End Function
	
	Rem
	bbdoc: Sets the simulation gravity.
	about: Gravity can go in any direction dictated by the vector @x,@y.
	End Rem
	Function SetGravity( x:Float, y:Float )
		phys.gravityX=x
		phys.gravityY=y
	End Function
	
	Rem
	bbdoc: The number of iterations used by the simulation.
	about: The higher the @value the more accurate and stiff the simulation.
	The lower the @value then the faster and softer the simulation.
	End Rem
	Function SetIterations( value:Int )
		Phys.i=value
	End Function
	
	Rem
	bbdoc: Set the upper and lower bounds of the simulation.
	End Rem
	Function SetBounds( lowerX:Float, lowerY:Float, upperX:Float, upperY:Float )
		phys.boundsMinX=Min(lowerX,upperX)
		phys.boundsMinY=Min(lowerY,upperY)
		phys.boundsMaxX=Max(lowerX,upperX)
		phys.boundsMaxY=Max(lowerY,upperY)
	End Function
	
	Rem
	bbdoc: Get the upper and lower bounds of the simulation.
	End Rem
	Function GetBounds( lowerX:Float Var, lowerY:Float Var, upperX:Float Var, upperY:Float Var )
		lowerX=phys.boundsMinX
		lowerY=phys.boundsMinY
		upperX=phys.boundsMaxX
		upperY=phys.boundsMaxY
	End Function
	
	Rem
	bbdoc: Set the friction of the simulations bounds.
	End Rem
	Function SetFriction( frict:Float )
		phys.friction=frict
	End Function
	
	Rem
	bbdoc: Activate or de-activate the use of the shifted grid for TMass occlusion.
	about: Set to true to use the shifted grid to occlude TMass objects.
	The shifted grid occlusion is not used by default.
	The shifted grid gives the best performance when used with extremely large simulations,
	and will actually be slower on very small simulations. For more information see #SetGridSize.
	EndRem
	Function UseShiftedGrid( value:Int )
		phys.gridon=value
	End Function
	
	Rem
	bbdoc: Set the size of the shifted grid.
	about: The shifted grid size can be in the range of 8-64.
	When using the shifted grid, setting the size of the grid too high can make the simulation unstable.
	To make sure that the grid is not set too high use the following equation. (maxBound-minBound)/diam.
	Where minBound and maxBound are the dimensions of the smallest side of the simulation area,
	and diam is the diameter(radius*2) of the largest TMass in the simulation.
	Even this equation will create a shifted grid size right on the limit
	so it is probably best to use this equation as an absolute maximum.
	EndRem
	Function SetGridSize( value:Int=8 )
		value=Min(MAX_GRID,Max(8,value))
		phys.gridsize=value
		phys.ratiox=phys.gridsize/(phys.boundsMaxX-phys.boundsMinX)
		phys.ratioy=phys.gridsize/(phys.boundsMaxY-phys.boundsMinY)
	End Function
	
	Rem
	bbdoc: Set the area of simulation to draw.
	about: @centerX and @centerY are the simulation coordinates to be mapped to the center of the screen.
	End Rem
	Function SetDrawCoords( centerX:Float, centerY:Float )
		phys.centerX=centerX
		phys.centerY=centerY
		UpdateTransform
	End Function
	
	Rem
	bbdoc: Set the zoom of the draw commands.
	about:
	End Rem
	Function SetDrawZoom( zoom:Float=1.0 )
		phys.drawScale=zoom
		UpdateTransform
	End Function
	
	Rem
	bbdoc: Sets the angle at which to draw the simulation.
	about: SetDrawRotation rotates all drawing commands in PhysLite.
	Use the drawing commands within PhysLite to draw in the corrrect simulation location.
	SetDrawRotation does not affect the ordinary Max2D drawing commands.
	End Rem
	Function SetDrawRotation( angle:Float=0 )
		phys.drawRot=angle
		UpdateTransform
	End Function
	
	Rem
	bbdoc: Updates the transform/rotation matrix.
	about: Under normal circamstances UpdateTransform will not be needed 
	as the transform/rotation matrix is update by any commands that would change it.
	However, UpdateTransform is required when the display dimensions are changed,
	and it becomes indespensable when using a MaxGUI canvas.
	End Rem
	Function UpdateTransform()
		phys.drawIX= Cos(phys.drawRot)*phys.drawScale
		phys.drawIY=-Sin(phys.drawRot)*phys.drawScale
		phys.drawJX= Sin(phys.drawRot)*phys.drawScale
		phys.drawJY= Cos(phys.drawRot)*phys.drawScale
		phys.drawOffsetX=-phys.centerX*phys.drawIX-phys.centerY*phys.drawIY+GraphicsWidth()*.5
		phys.drawOffsetY=-phys.centerX*phys.drawJX-phys.centerY*phys.drawJY+GraphicsHeight()*.5
	End Function
	
	Rem
	bbdoc: Gets the mouse X coordinate.
	returns: Mouse X coordinate.
	about: MouseX returns the simulation X coordinate of the mouse cursor.
	End Rem
	Function MouseX:Float()
		Local mx:Float=.MouseX()
		Local my:Float=.MouseY()
		
		Return ((mx-phys.drawOffsetX)*phys.drawIX+(my-phys.drawOffsetY)*-phys.drawIY)/(phys.drawScale*phys.drawScale)
		
	End Function
	
	Rem
	bbdoc: Gets the mouse Y coordinate.
	returns: Mouse Y coordinate.
	about: MouseY returns the simulation Y coordinate of the mouse cursor.
	End Rem
	Function MouseY:Float()
		Local mx:Float=.MouseX()
		Local my:Float=.MouseY()
		
		Return  ((mx-phys.drawOffsetX)*-phys.drawJX+(my-phys.drawOffsetY)*phys.drawJY)/(phys.drawScale*phys.drawScale)
		
	End Function
	
	Rem
	bbdoc: Gets the simulation X coordinate of a screen x,y location.
	returns: Simulation X coordinate.
	about: .
	End Rem
	Function SimX:Float( x:Float, y:Float )
		Return ((x-phys.drawOffsetX)*phys.drawIX+(y-phys.drawOffsetY)*-phys.drawIY)/(phys.drawScale*phys.drawScale)
	End Function
	
	Rem
	bbdoc: Gets the simulation Y coordinate of a screen x,y location.
	returns: Simulation Y coordinate.
	about: .
	End Rem
	Function SimY:Float( x:Float, y:Float )
		Return ((x-phys.drawOffsetX)*-phys.drawJX+(y-phys.drawOffsetY)*phys.drawJY)/(phys.drawScale*phys.drawScale)
	End Function
	
	Rem
	bbdoc: Draws an image.
	about: Draws an image at simulation coordinates x,y and with the correct rotation.
	End Rem
	Function DrawImage( image:TImage, x:Float, y:Float, frame:Int=0 )
		Local sclx:Float,scly:Float,rot:Float
		GetScale sclx,scly
		rot=GetRotation()
		SetScale phys.drawScale*sclx,phys.drawScale*scly
		SetRotation rot+phys.drawRot
		.DrawImage image,ScreenX(x,y),ScreenY(x,y),frame
		SetScale sclx,scly
		SetRotation rot
	End Function
	
	Rem
	bbdoc: Draws a polygon.
	about: Draws a polygon at simulation coordinates x,y and with the correct rotation.
	End Rem
	Function DrawPoly( xy:Float[], x:Float=0.0, y:Float=0.0 )
		Local sclx:Float,scly:Float
		GetScale sclx,scly
		SetScale phys.drawScale,phys.drawScale
		
		Local ox:Float,oy:Float
		GetOrigin ox,oy
		SetOrigin ScreenX(x,y)+ox,ScreenY(x,y)+oy
		
		Local rot:Float=GetRotation()
		SetRotation phys.drawRot+rot
		
		.DrawPoly xy
		
		SetOrigin ox,oy
		SetRotation rot
		SetScale sclx,scly
	End Function
	
	Rem
	bbdoc: Draws a line.
	about: Draws a line at the correct simulation coordinates.
	End Rem
	Function DrawLine( x:Float, y:Float, x2:Float, y2:Float )
		.DrawLine TPhysics.ScreenX(x,y),TPhysics.ScreenY(x,y),TPhysics.ScreenX(x2,y2),ScreenY(x2,y2)
	End Function
	
	Rem
	bbdoc: Draws an oval.
	about: Draws an oval at simulation coordinates x,y and with the correct rotation.
	End Rem
	Function DrawOval( x:Float, y:Float, width:Float, height:Float )
		Local rot:Float=GetRotation()
		SetRotation phys.drawRot+rot
		
		.DrawOval TPhysics.ScreenX(x,y),TPhysics.ScreenY(x,y),width*phys.drawScale,height*phys.drawScale
		
		SetRotation rot
	End Function
	
	Rem
	bbdoc: Draws a rectangle.
	about: Draws a rectangle at simulation coordinates x,y and with the correct rotation.
	End Rem
	Function DrawRect( x:Float, y:Float, width:Float, height:Float )
		Local rot:Float=GetRotation()
		SetRotation phys.drawRot+rot
		
		.DrawRect TPhysics.ScreenX(x,y),TPhysics.ScreenY(x,y),width*phys.drawScale,height*phys.drawScale
		
		SetRotation rot
	End Function
	
	Rem
	bbdoc: Gets the screen x coordinate.
	returns: The screen x coordinate of simulation x,y coordinate.
	End Rem
	Function ScreenX:Float( valueX:Float, valueY:Float )
		Return phys.drawOffsetX+valueX*phys.drawIX+valueY*phys.drawIY'phys.drawScale
	End Function
	
	
	Rem
	bbdoc: Gets the screen y coordinate.
	returns: The screen y coordinate of simulation x,y coordinate.
	End Rem
	Function ScreenY:Float( valueX:Float, valueY:Float )
		Return phys.drawOffsetY+valueX*phys.drawJX+valueY*phys.drawJY'phys.drawScale
	End Function
	
	Rem
	bbdoc: Update simulation
	about: Process the physics iterations and responses.
	#Update will #TColEvent.Flush the event collision queue.
	End Rem
	Function Update()
		'Move the masses
		For Local count:Int=0 Until TMass.aNext
			If TMass.array[count]
				Local m:TMass=TMass.array[count]
				If Not m.frozen
					Local tmpX:Float=m.posX
				Local tmpY:Float=m.posY
					m.posX:+m.posX-m.oldPosX+phys.gravityX
					m.posY:+m.posY-m.oldPosY+phys.gravityY
					m.oldPosX=tmpX
					m.oldPosY=tmpY
				EndIf
				m.AddToGrid
			EndIf
		Next
		
		TColEvent.Flush
		
		'Relax constraints
		For Local i:Int=1 To Phys.i
			'Stick constraints
			For Local count:Int=0 Until TConstraint.aNext			
				If TConstraint.array[count]
					TConstraint.array[count].Satisfy()
				EndIf
			Next
			
			'Collision constraints
			For Local m1Count:Int=0 Until TMass.aNext
				If TMass.array[m1Count]
					Local m1:TMass=TMass.array[m1Count]
					'World bounds
					If m1.posX<phys.boundsMinX+m1.radius
						Local pen:Float=phys.boundsMinX+m1.radius-m1.posX
						m1.posX=phys.boundsMinX+m1.radius
						TColEvent.Emit TColEvent.Create(m1,phys,phys.boundsMinX,m1.posY,pen)
						'Friction
						Local frict:Float=pen*(m1.friction+phys.friction)*0.5
						Local tanVel:Float=m1.posY-m1.oldPosY
						If frict>Abs(tanVel)
							m1.oldPosY=m1.posY
						Else
							m1.oldPosY:+frict*Sgn(tanVel)
						EndIf
					ElseIf m1.posX>phys.boundsMaxX-m1.radius
						Local pen:Float=m1.posX-(phys.boundsMaxX-m1.radius)
						m1.posX=phys.boundsMaxX-m1.radius
						TColEvent.Emit TColEvent.Create(m1,phys,phys.boundsMaxX,m1.posY,pen)
						'Friction
						Local frict:Float=pen*(m1.friction+phys.friction)*0.5
						Local tanVel:Float=m1.posY-m1.oldPosY
						If frict>Abs(tanVel)
							m1.oldPosY=m1.posY
						Else
							m1.oldPosY:+frict*Sgn(tanVel)
						EndIf
					EndIf
					If m1.posY<phys.boundsMinY+m1.radius
						Local pen:Float=phys.boundsMinY+m1.radius-m1.posY
						m1.posY=phys.boundsMinY+m1.radius
						TColEvent.Emit TColEvent.Create(m1,phys,m1.posX,phys.boundsMinY,pen)
						'Friction
						Local frict:Float=pen*(m1.friction+phys.friction)*0.5
						Local tanVel:Float=m1.posX-m1.oldPosX
						If frict>Abs(tanVel)
							m1.oldPosX=m1.posX
						Else
							m1.oldPosX:+frict*Sgn(tanVel)
						EndIf
					ElseIf m1.posY>phys.boundsMaxY-m1.radius
						Local pen:Float=m1.posY-(phys.boundsMaxY-m1.radius)
						m1.posY=phys.boundsMaxY-m1.radius
						TColEvent.Emit TColEvent.Create(m1,phys,m1.posX,phys.boundsMaxY,pen)
						'Friction
						Local frict:Float=pen*(m1.friction+phys.friction)*0.5
						Local tanVel:Float=m1.posX-m1.oldPosX
						If frict>Abs(tanVel)
							m1.oldPosX=m1.posX
						Else
							m1.oldPosX:+frict*Sgn(tanVel)
						EndIf
					EndIf
					'Zone collisions
					If Not m1.frozen And m1.colGroup<>0
						For Local count:Int=0 Until TZone.aNext
							If TZone.array[count]
								Local z:TZone=TZone.array[count]
								If z.colGroup<>0 And m1.colGroup<>z.colGroup Then z.Collide(m1)
							EndIf
						Next
					EndIf
					'Mass collisions
					If phys.gridon And m1.colGroup<>0
						m1.AddToGrid
						For Local xcount:Int=m1.gridx-1 To m1.gridx+1
							For Local ycount:Int=m1.gridy-1 To m1.gridy+1
								If xcount>=0 And xcount<phys.gridsize And ycount>=0 And ycount<phys.gridsize
									For Local m2:TMass=EachIn phys.shiftgrid[xcount,ycount]
										If m2.colGroup<>0 And m1.colGroup<>m2.colGroup Then m1.Collide(m2)
									Next
								EndIf
							Next
						Next
					ElseIf m1.colGroup<>0
						For Local m2Count:Int=m1Count+1 Until TMass.aNext
							If TMass.array[m2Count]
								Local m2:TMass=TMass.array[m2Count]
								If m2.colGroup<>0 And m1.colGroup<>m2.colGroup Then m1.Collide(m2)
							EndIf
						Next
					EndIf
				EndIf
			Next
			
		Next
		
	End Function
	
	Method New()
		'Init manged arrays
		TMass.array=New TMass[1024]
		TMass.aNext=0
		TMass.stack=New Int[1024]
		TMass.sNext=0
		
		TConstraint.array=New TConstraint[1024]
		TConstraint.aNext=0
		TConstraint.stack=New Int[1024]
		TConstraint.sNext=0
		
		TZone.array=New TZone[1024]
		TZone.aNext=0
		TZone.stack=New Int[1024]
		TZone.sNext=0
		
		TAdvBody.array=New TAdvBody[1024]
		TAdvBody.aNext=0
		TAdvBody.stack=New Int[1024]
		TAdvBody.sNext=0
		
		'Init shifted grid
		For Local ycount:Int=0 To MAX_GRID-1
			For Local xcount:Int=0 To MAX_GRID-1
				shiftgrid[xcount,ycount]=New TList
			Next
		Next
		
		'Assign physics context
		phys=Self
	End Method
	
End Type

Rem
bbdoc: Mass object.
End Rem
Type TMass
	Global array:TMass[]
	Global aNext:Int=0
	Global stack:Int[]
	Global sNext:Int=0
	
	Field name:String
	
	Field posX:Float, posY:Float
	Field oldPosX:Float, oldPosY:Float
	Field mass:Float
	Field radius:Float
	Field colGroup:Float
	Field friction:Float	'The friction coefficient of the mass
	Field frozen:Int=False
	
	Field gridlink:TLink	'Masses link in shifted grid
	Field gridx:Int, gridy:Int
	Field refCounter:Int=0
	
	Field aIndex:Int=-1
	
	Field abID:Int=-1	'Index of masses advbody
	Field index:Int=-1	'index of mass within advbody
	
	Rem
	bbdoc: Creates a mass object.
	returns: Created TMass.
	about: The Create function creates a TMass to be used by the simulation.
	The TMass does not become active until it is assigned to an active TConstraint.
	The parameter @colGroup is the masses collision group.
	A value of -1 will set the masses collision group to the next unique value.
	If two collidable objects have the same collision group then they will @not collide with each other.
	If @colGroup is set to 0 then the mass will @not collide with any other object.
	End Rem
	Function Create:TMass( x:Float, y:Float, mass:Float, radius:Float, friction:Float, colGroup:Int=-1 )
		
		Local m:TMass=New TMass
		m.name="Mass"
		m.posX=x
		m.posY=y
		m.oldPosX=x
		m.oldPosY=y
		m.mass=mass
		m.radius=radius
		m.friction=Max(0,Min(1,friction))
		
		If colGroup=-1
			m.colGroup=TPhysics.cgCounter
			TPhysics.cgCounter:-1
		Else
			m.colGroup=colGroup
		EndIf
		
		Return m
		
	End Function
	
	Rem
	bbdoc: Counts the number of active #TMass objects.
	returns: The number of active #TMass objects.
	about: CountActive only counts #TMass objects that are associated with an active #TConstraint.
	End Rem
	Function CountActive:Int()
		Return aNext-sNext
	End Function
	
	Rem
	bbdoc: Maps one function to all active TMass objects.
	about: ApplyFunc is the function that will be run for every active TMass in the simulation.
	End Rem
	Function ApplyToActive( ApplyFunc( mass:TMass ) )
		For Local count:Int=0 Until aNext
			If array[count]
				ApplyFunc array[count]
			EndIf 
		Next
	End Function
	
	Rem
	bbdoc: Creates a copy of the mass.
	returns: TMass Copy.
	End Rem
	Method Copy:TMass()
		
		Local m:TMass=New TMass
		m.name=name
		m.posX=posX
		m.posY=posY
		m.oldPosX=oldPosX
		m.oldPosY=oldPosY
		m.mass=mass
		m.radius=radius
		m.friction=friction
		m.colGroup=colGroup
		m.frozen=frozen
		
		m.index=index
		
		Return m
		
	End Method
	
	Rem
	bbdoc: Sets masses name.
	End Rem
	Method SetName( n:String )
		name=n
	End Method
	
	Rem
	bbdoc: Gets masses name.
	End Rem
	Method GetName:String()
		Return name
	End Method
	
	Rem
	bbdoc: Moves TMass relative to it's current position.
	about: When using #Move it's important to note that the masses velocity will also be altered by the same amount.
	To move a masses without affecting it's velocity then use #Translate.
	A frozen, see #Freeze, TMass cannot be moved.
	End Rem
	Method Move( x:Float, y:Float )
		If frozen Then Return
		posX:+x
		posY:+y
	End Method
	
	Rem
	bbdoc: Translates TMass.
	about: #Translate works the same as #Move except the masses velocity remains the same.
	#Translate can be used on a frozen mass.
	End Rem
	Method Translate( x:Float, y:Float )
		posX:+x
		posY:+y
		oldPosX:+x
		oldPosY:+y
	End Method
	
	Rem
	bbdoc: Sets masses absolute position.
	about: Positioning a mass also resets it's velocity.
	#Position may be used on a frozen mass.
	End Rem
	Method Position( x:Float, y:Float )
		posX=x
		posY=y
		oldPosX=x
		oldPosY=y
	End Method
	
	Rem
	bbdoc: Gets masses current X coordinate.
	returns: masses X coordinate.
	End Rem
	Method GetPositionX:Float()
		Return posX
	End Method
	
	Rem
	bbdoc: Gets masses current Y coordinate.
	returns: Masses Y coordinate.
	End Rem
	Method GetPositionY:Float()
		Return posY
	End Method
	
	Rem
	bbdoc: Sets @x and @y to masses position.
	End Rem
	Method GetPosition( x:Float Var, y:Float Var )
		x=posX
		y=posY
	End Method
	
	Rem
	bbdoc: Gets masses current X coordinate.
	returns: masses X coordinate.
	about: Short hand version of #GetPositionX.
	End Rem
	Method GetX:Float()
		Return posX
	End Method
	
	Rem
	bbdoc: Gets masses current Y coordinate.
	returns: Masses Y coordinate.
	about: Short hand version of #GetPositionY.
	End Rem
	Method GetY:Float()
		Return posY
	End Method
	
	Rem
	bbdoc: Sets the current mass of the TMass object.
	about: Mass should always be positive.
	End Rem
	Method SetMass( m:Float )
		mass=m
	End Method
	
	Rem
	bbdoc: Gets the current mass of the TMass object.
	returns: Current mass.
	End Rem
	Method GetMass:Float()
		Return mass
	End Method
	
	Rem
	bbdoc: Sets the collision radius of the mass
	End Rem
	Method SetRadius( r:Float )
		radius=r
	End Method
	
	Rem
	bbdoc: Gets the collision radius of the mass.
	End Rem
	Method GetRadius:Float()
		Return radius
	End Method
	
	Rem
	bbdoc: Sets masses friction.
	about: Friction should always be positive.
	End Rem
	Method SetFriction( f:Float )
		friction=Max(0,Min(1,f))
	End Method
	
	Rem
	bbdoc: Gets the current friction coefficient of the TMass object.
	returns: Current friction coefficient.
	End Rem
	Method GetFriction:Float()
		Return friction
	End Method
	
	Rem
	bbdoc: Freezes TMass
	about: Freezing a TMass makes it immovable during the simulation.
	The following methods do @not affect a frozen TMass, #Move, #Translate, #SetVelocity and #AddVelocity.
	End Rem
	Method Freeze()
		oldPosX=posX
		oldPosY=posY
		frozen=True
	End Method
	
	Rem
	bbdoc: Thaws TMass
	about: #Thaw unfreezes the TMass.
	End Rem
	Method Thaw()
		frozen=False
	End Method
	
	Rem
	return: Returns true if mass is frozen.
	EndRem
	Method IsFrozen:Int()
		Return frozen
	End Method
	
	Rem
	bbdoc: Sets masses velocity.
	about: Sets the masses velocity.
	End Rem
	Method SetVelocity( x:Float, y:Float )
		If frozen Then Return
		oldPosX=posX-x
		oldPosY=posY-y
	End Method
	
	Rem
	bbdoc: Alters masses velocity.
	about: #AddVelocity differs from #SetVelocity in that the masses current velocity is added to instead of replaced by @X and @Y.
	End Rem
	Method AddVelocity( x:Float, y:Float )
		If frozen Then Return
		oldPosX:-x
		oldPosY:-y
	End Method
	
	Rem
	bbdoc: Gets TMass objects X velocity
	returns: Masses x velocity.
	End Rem
	Method GetVelocityX:Float()
		Return posX-oldPosX
	End Method
	
	Rem
	bbdoc: Gets TMass objects Y velocity
	returns: Masses y velocity
	End Rem
	Method GetVelocityY:Float()
		Return posY-oldPosY
	End Method
	
	Rem
	bbdoc: Gets TMass objects X velocity
	returns: Masses x velocity.
	about: Short hand version of #GetVelocityX
	End Rem
	Method GetVX:Float()
		Return posX-oldPosX
	End Method
	
	Rem
	bbdoc: Gets TMass objects Y velocity
	returns: Masses y velocity
	about: Short hand version of #GetVelocityY
	End Rem
	Method GetVY:Float()
		Return posY-oldPosY
	End Method
	
	Rem
	bbdoc: Gets the magnitude of the TMass objects velocity vector
	End Rem
	Method GetVelocity:Float()
		Return Sqr( (posX-oldPosX)*(posX-oldPosX) + (posY-oldPosY)*(posY-oldPosY) )
	End Method
	
	Rem
	bbdoc: Returns masses collision group
	returns: Collision group
	End Rem
	Method GetColGroup:Int()
		Return colGroup
	End Method
	
	Rem
	bbdoc: Sets masses collision group
	about: A value of -1 will set the masses collision group to the next unique value.
	If two collidable objects have the same collision group then they will @not collide with each other.
	If @colGroup is set to 0 then the mass will @not collide with any other object.
	End Rem
	Method SetColGroup( value:Int=-1 )
		If value=-1
			colGroup=TPhysics.cgCounter
			TPhysics.cgCounter:-1
		Else
			colGroup=value
		EndIf
	End Method
	
	Rem
	bbdoc: Checks if mass is currently active in the simulation
	returns: True if mass is active
	End Rem
	Method IsActive:Int()
		If refCounter>0 Then Return True Else Return False
	End Method
	
	Rem
	bbdoc: Gets the index of the mass
	returns: The index of the mass
	about: GetIndex returns -1 if the mass isn't associated with an TAdvBody
	End Rem
	Method GetIndex:Int()
		Return index
	End Method
	
	Rem
	bbdoc: Gets the #TAdvBody that the mass is associated with.
	returns: The #TAdvBody of the mass
	about: GetAdvBody returns #null if the mass isn't associated with an TAdvBody.
	End Rem
	Method GetAdvBody:TAdvBody()
		If abID<0 Or abID>=TAdvBody.aNext
			Return Null
		Else
			Return TAdvBody.array[abID]
		EndIf
	End Method
	
	Rem
	bbdoc: Draws the TMass.
	End Rem
	Method Draw()
		Local rot:Float=GetRotation()
		SetRotation 0
		
		Local diam:Float=radius*2
		TPhysics.DrawOval posX-radius,posY-radius,diam,diam
		
		SetRotation rot
	End Method
	
	'WARNING for internal use only.
	Method AddRef()
		If refCounter=0
			If sNext>0
				sNext:-1
				aIndex=stack[sNext]
			Else
				aIndex=aNext
				aNext:+1
				If aNext=array.length
					array=array[..aNext*2]
					stack=stack[..aNext*2]
				EndIf
			EndIf
			array[aIndex]=Self
			AddToGrid
		EndIf
		refCounter:+1
	End Method
	
	'WARNING for internal use only.
	Method AddToGrid()
		If gridlink Then gridlink.Remove
		gridx=Floor(posX*phys.ratiox)
		gridy=Floor(posY*phys.ratioy)
		gridx=Min(Max(gridx,0),phys.gridsize-1)
		gridy=Min(Max(gridy,0),phys.gridsize-1)
		gridlink=phys.shiftgrid[gridx,gridy].AddLast(Self)
	End Method
	
	'WARNING for internal use only
	Method SubRef()
		refCounter:-1
		If refCounter=0
			stack[sNext]=aIndex
			sNext:+1
			array[aIndex]=Null
			aIndex=-1
			If aNext=sNext
				aNext=0
				sNext=0
			EndIf
			gridlink.Remove
		EndIf
	End Method
	
	'WARNING for internal use only
	Method Collide( m:TMass )
		If frozen And m.frozen
			Return
		Else
			Local restLength:Float=radius+m.radius
			If Abs(m.posX-posX)>restLength Or Abs(m.posY-posY)>restLength Then Return
			Local penVectX:Float=posX-m.posX
			Local penVectY:Float=posY-m.posY
			Local dist:Float = Sqr( penVectX*penVectX + penVectY*penVectY)
			If dist<PHYS_MIN_FLOAT Then dist=PHYS_MIN_FLOAT
			Local penNormX:Float=penVectX/dist
			Local penNormY:Float=penVectY/dist
			Local penMag:Float=dist-restLength
			
			If penMag<0
				penVectX=penNormX*penMag
				penVectY=penNormY*penMag
				Local frictCoef:Float=-penMag*(friction+m.friction)*0.5
				
				If m.frozen
					
					posX:-penVectX
					posY:-penVectY
					
					'Friction
					Local velVectX:Float=posX-oldPosX
					Local velVectY:Float=posY-oldPosY
					Local vel:Float=Sqr(velVectX*velVectX+velVectY*velVectY)
					Local angle:Float=ATan2(velVectY,velVectX)-ATan2(penNormY,penNormX)
					Local tanVel:Float=Cos(angle)*vel
					Local tanVectX:Float=penNormX*tanVel
					Local tanVectY:Float=penNormY*tanVel
					tanVectX=velVectX-tanVectX
					tanVectY=velVectY-tanVectY
					tanVel=Sqr(tanVectX*tanVectX+tanVectY*tanVectY)
					
					If frictCoef>=tanVel
						oldPosX:+tanVectX
						oldPosY:+tanVectY
					Else
						oldPosX:+(tanVectX/tanVel)*frictCoef
						oldPosY:+(tanVectY/tanVel)*frictCoef
					EndIf
					
				ElseIf frozen
					
					m.posX:+penVectX
					m.posY:+penVectY
					
					'Friction
					Local velVectX:Float=m.posX-m.oldPosX
					Local velVectY:Float=m.posY-m.oldPosY
					Local vel:Float=Sqr(velVectX*velVectX+velVectY*velVectY)
					Local angle:Float=ATan2(velVectY,velVectX)-ATan2(penNormY,penNormX)
					Local tanVel:Float=Cos(angle)*vel
					Local tanVectX:Float=penNormX*tanVel
					Local tanVectY:Float=penNormY*tanVel
					tanVectX=velVectX-tanVectX
					tanVectY=velVectY-tanVectY
					tanVel=Sqr(tanVectX*tanVectX+tanVectY*tanVectY)
					
					If frictCoef>=tanVel
						m.oldPosX:+tanVectX
						m.oldPosY:+tanVectY
					Else
						m.oldPosX:+(tanVectX/tanVel)*frictCoef
						m.oldPosY:+(tanVectY/tanVel)*frictCoef
					EndIf
					
				Else
					
					Local w:Float = mass + m.mass
					
					posX:-penVectX*(m.mass/w)
					posY:-penVectY*(m.mass/w)
					
					m.posX:+penVectX*(mass/w)
					m.posY:+penVectY*(mass/w)
					
					'Friction
					Local velVectX:Float=posX-oldPosX
					Local velVectY:Float=posY-oldPosY
					Local vel:Float=Sqr(velVectX*velVectX+velVectY*velVectY)
					Local angle:Float=ATan2(velVectY,velVectX)-ATan2(penNormY,penNormX)
					Local tanVel:Float=Cos(angle)*vel
					Local tanVectX:Float=penNormX*tanVel
					Local tanVectY:Float=penNormY*tanVel
					tanVectX=velVectX-tanVectX
					tanVectY=velVectY-tanVectY
					
					Local mvelVectX:Float=m.posX-m.oldPosX
					Local mvelVectY:Float=m.posY-m.oldPosY
					Local mvel:Float=Sqr(mvelVectX*mvelVectX+mvelVectY*mvelVectY)
					Local mangle:Float=ATan2(mvelVectY,mvelVectX)-ATan2(penNormY,penNormX)
					Local mtanVel:Float=Cos(mangle)*mvel
					Local mtanVectX:Float=penNormX*mtanVel
					Local mtanVectY:Float=penNormY*mtanVel
					mtanVectX=mvelVectX-mtanVectX
					mtanVectY=mvelVectY-mtanVectY
					
					tanVectX:-(mtanVectX+tanVectX)*0.5
					tanVectY:-(mtanVectY+tanVectY)*0.5
					tanVel=Sqr(tanVectX*tanVectX+tanVectY*tanVectY)
					
					If frictCoef>=tanVel
						oldPosX:+tanVectX
						oldPosY:+tanVectY
						m.oldPosX:-tanVectX
						m.oldPosY:-tanVectY
					Else
						oldPosX:+(tanVectX/tanVel)*frictCoef
						oldPosY:+(tanVectY/tanVel)*frictCoef
						m.oldPosX:-(tanVectX/tanVel)*frictCoef
						m.oldPosY:-(tanVectY/tanVel)*frictCoef
					EndIf
Rem					
					If frictCoef>=tanVel
						PosX:-tanVectX
						PosY:-tanVectY
						m.PosX:+tanVectX
						m.PosY:+tanVectY
					Else
						PosX:-(tanVectX/tanVel)*frictCoef
						PosY:-(tanVectY/tanVel)*frictCoef
						m.PosX:+(tanVectX/tanVel)*frictCoef
						m.PosY:+(tanVectY/tanVel)*frictCoef
					EndIf
EndRem					
				EndIf
				TColEvent.Emit( TColEvent.Create(Self,m,(posX+m.posX)*.5,(posY+m.posY)*.5,-penMag) )
			EndIf
		EndIf
	End Method
End Type

Rem
bbdoc: Constraint object.
End Rem
Type TConstraint
	Global array:TConstraint[]
	Global aNext:Int=0
	Global stack:Int[]
	Global sNext:Int=0
	
	Field name:String
	
	Field m:TMass[2]
	Field restLength:Float
	Field stiffness:Float
	
	Field refCounter:Int=0
	
	Field aIndex:Int=-1
	
	Field abID:Int=-1
	Field index:Int=-1
	
	Rem
	bbdoc: Creates a new TConstraint object.
	returns: Created TConstraint.
	about: A constraint is an invisible conection between two TMasses.
	@restLength is the length of the constraint.
	Setting @restLength to -1 results in @restLength being calculated based on the current distance between the two masses.
	End Rem
	Function Create:TConstraint( m1:TMass, m2:TMass, restLength:Float=-1, stiffness:Float=1.0 )
		
		Local c:TConstraint=New TConstraint
		
		c.m[0]=m1
		c.m[1]=m2
		If restLength<=0.0
			c.restLength=Sqr((m1.posX-m2.posX)*(m1.posX-m2.posX)+(m1.posY-m2.posY)*(m1.posY-m2.posY))
		Else
			c.restLength=restLength
		EndIf
		c.stiffness=stiffness
		
		c.name="Con"
		
		Return c
		
	End Function
	
	Rem
	bbdoc: Counts the number of active #TConstraints.
	returns: The number of active #TConstraints.
	about: CountActive only counts #TConstraints that are associated with a #TBody.
	End Rem
	Function CountActive:Int()
		Return aNext-sNext
	End Function
	
	Rem
	bbdoc: Maps one function to all active TConstraints.
	about: ApplyFunc is the function that will be run for every active TConstraint in the simulation.
	End Rem
	Function ApplyToActive( ApplyFunc( con:TConstraint ) )
		For Local count:Int=0 Until aNext
			If array[count]
				ApplyFunc array[count]
			EndIf 
		Next
	End Function
	
	Rem
	bbdoc: Sets constraints name.
	End Rem
	Method SetName( n:String )
		name=n
	End Method
	
	Rem
	bbdoc: Gets constraints name.
	returns: Constraints name.
	End Rem
	Method GetName:String()
		Return name
	End Method
	
	Rem
	bbdoc: Assigns constraint to a TBody.
	about: A constraint must be assigned to a TBody before it becomes active.
	End Rem
	Method Assign( b:TBody )
		If Not b.conList.Contains(Self)
			b.conList.AddLast(Self)
			AddRef
		EndIf
	End Method
	
	Rem
	bbdoc: Frees constraint from TBody.
	about: @Note: A constraint can be assigned to more than one TBody.
	Freeing a constraint from one TBody will not free it from any others it is assigned to.
	End Rem
	Method Free( b:TBody )
		Local link:TLink=b.conList.FindLink(Self)
		If link
			SubRef
			link.Remove
		EndIf
	End Method
	
	Rem
	bbdoc: Gets the first mass associated with the constraint
	about: Each constraint has two masses associated with it.
	#GetMass1 returns the first, to get the second use #GetMass2.
	End Rem
	Method GetMass1:TMass()
		Return m[0]
	End Method
	
	Rem
	bbdoc: Gets the second mass associated with the constraint
	about: Each constraint has two masses associated with it.
	#GetMass2 returns the second, to get the first use #GetMass1.
	End Rem
	Method GetMass2:TMass()
		Return m[1]
	End Method
	
	Rem
	bbdoc: Sets the constraints rest length.
	about: The constraints rest length is the length the constraint attempts to maintain.
	End Rem
	Method SetRestLength( length:Float=-1 )
		If length<=0.0
			restLength=Sqr((m[0].posX-m[1].posX)*(m[0].posX-m[1].posX)+(m[0].posY-m[1].posY)*(m[0].posY-m[1].posY))
		Else
			restLength=length
		EndIf
	End Method
	
	Rem
	bbdoc: Gets the constraints rest length.
	returns: Rest length.
	about: @Note: The current length of the constraint may differ from it's intended rest length.
	To get the constraints actual length use #GetLength.
	End Rem
	Method GetRestLength:Float()
		Return restLength
	End Method
	
	Rem
	bbdoc: Sets constraints stiffness
	about: Stiffnes should be within the range 0.0 to 1.0.
	End Rem
	Method SetStiffness( stiff:Float )
		stiffness=Max(0,Min(1,stiff))
	End Method
	
	Rem
	bbdoc: Gets current stiffness.
	returns: Current stiffness.
	End Rem
	Method GetStiffness:Float()
		Return stiffness
	End Method
	
	Rem
	bbdoc: Gets the current angle of the constraint.
	returns: Current angle.
	about: This is the angle from mass1 to mass2.
	End Rem
	Method GetAngle:Float()
		Return ATan2(m[1].posY-m[0].posY,m[1].posX-m[0].posX)
	End Method
	
	Rem
	bbdoc: Gets the constraints actual length.
	returns: Length.
	about: #GetLength is used to find the current length of the constraint.
	End Rem
	Method GetLength:Float()
		Return Sqr((m[0].posX-m[1].posX)*(m[0].posX-m[1].posX)+(m[0].posY-m[1].posY)*(m[0].posY-m[1].posY))
	End Method
	
	Rem
	bbdoc: Gets the current stress of the constraint.
	returns: Stress ratio.
	about: A returned value of 1.0 means the constraint is under no stress and is at rest.
	A value>1.0 means the constraint is being stretched.
	A value<1.0 means the constraint is being compressed.
	#GetStress will never return a negative result.
	End Rem
	Method GetStress:Float()
		Return GetLength()/restLength
	End Method
	
	Rem
	bbdoc: Checks if constraint is currently active in the simulation
	returns: True if constraint is active
	End Rem
	Method IsActive:Int()
		If refCounter>0 Then Return True Else Return False
	End Method
	
	Rem
	bbdoc: Gets the index of the constraint
	returns: The index of the constraint
	about: GetIndex returns -1 if the constraint isn't associated with an TAdvBody
	End Rem
	Method GetIndex:Int()
		Return index
	End Method
	
	Rem
	bbdoc: Gets the #TAdvBody that the constraint is associated with.
	returns: The #TAdvBody of the constraint
	about: GetAdvBody returns #null if the constraint isn't associated with an TAdvBody.
	End Rem
	Method GetAdvBody:TAdvBody()
		If abID<0 Or abID>=TAdvBody.aNext
			Return Null
		Else
			Return TAdvBody.array[abID]
		EndIf
	End Method
	
	Rem
	bbdoc: Draws the TConstraint.
	End Rem
	Method Draw()
		TPhysics.DrawLine m[0].posX,m[0].posy,m[1].posX,m[1].posY
	End Method
	
	'WARNING for internal use only
	Method Satisfy()
		If m[0].frozen And m[1].frozen
			Return
		Else
			Local dx:Float = m[0].posX - m[1].posX
			Local dy:Float = m[0].posY - m[1].posY
			Local dist:Float = Sqr( dx*dx + dy*dy)
			If dist<PHYS_MIN_FLOAT Then dist=PHYS_MIN_FLOAT
			
			If m[0].frozen
				m[1].posX :+ ((dx / dist) * ((dist - restLength) * stiffness))
				m[1].posY :+ ((dy / dist) * ((dist - restLength) * stiffness))
			ElseIf m[1].frozen
				m[0].posX :- ((dx / dist) * ((dist - restLength) * stiffness))
				m[0].posY :- ((dy / dist) * ((dist - restLength) * stiffness))
			Else
				Local w:Float = m[0].mass + m[1].mass
				
				m[0].posX :- ((dx / dist) * ((dist - restLength) * stiffness)) * (m[1].mass / w)
				m[0].posY :- ((dy / dist) * ((dist - restLength) * stiffness)) * (m[1].mass / w)
				
				m[1].posX :+ ((dx / dist) * ((dist - restLength) * stiffness)) * (m[0].mass / w)
				m[1].posY :+ ((dy / dist) * ((dist - restLength) * stiffness)) * (m[0].mass / w)
			EndIf
			
		EndIf
	End Method
	
	'WARNING for internal use only
	Method AddRef()
		If refCounter=0
			m[0].AddRef
			m[1].AddRef
			If sNext>0
				sNext:-1
				aIndex=stack[sNext]
			Else
				aIndex=aNext
				aNext:+1
				If aNext=array.length
					array=array[..aNext*2]
					stack=stack[..aNext*2]
				EndIf
			EndIf
			array[aIndex]=Self
		EndIf
		refCounter:+1
	End Method
	
	'WARNING for internal use only
	Method SubRef()
		refCounter:-1
		If refCounter=0
			m[0].SubRef
			m[1].SubRef
			stack[sNext]=aIndex
			sNext:+1
			array[aIndex]=Null
			aIndex=-1
			If aNext=sNext
				aNext=0
				sNext=0
			EndIf
		EndIf
	End Method
	
End Type

Rem
bbdoc: TBody object.
End Rem
Type TBody
	Field name:String="Body"
	
	Field conList:TList=New TList
	
	Rem
	bbdoc: Sets bodies name.
	End Rem
	Method SetName( n:String )
		name=n
	End Method
	
	Rem
	bbdoc: Gets bodies name.
	End Rem
	Method GetName:String()
		Return name
	End Method
	
	Rem
	bbdoc: Gets bodies constraints
	returns: list of constraints
	about: Creates a list containing all the Constraints associsted with the TBody object.
	End Rem
	Method GetConstraints:TList()
		Return conList.Copy()
	End Method
	
	Rem
	bbdoc: Frees all constraints from TBody.
	about: Frees all constraints that are currently assigned to the TBody.
	Use #Free when a body is no longer needed.
	End Rem
	Method Free()
		For Local c:TConstraint=EachIn conList
			c.SubRef
		Next
		conList.Clear
	End Method
	
	Rem
	bbdoc: Deprecated: Use #Free instead.
	about: Deprecated: Use #Free instead.
	End Rem
	Method Clear()
		For Local c:TConstraint=EachIn conList; c.SubRef; Next
		conList.Clear
	End Method
	
	Method Delete()
		Clear
	End Method
	
End Type

Rem
bbdoc: TRig object
End Rem
Type TRig
	
	Field name:String
	
	Field image:TImage
	Field mass:TMass
	Field con:TConstraint
	Field angle:Float
	Field scaleX:Float=1.0,scaleY:Float=1.0
	Field red:Float=1.0,green:Float=1.0,blue:Float=1.0,alpha:Float=1.0
	Field blend:Int=MASKBLEND
	
	Field abID:Int=-1
	Field index:Int=-1
	
	Rem
	bbdoc: Creates a new TRig
	returns: Created TRig
	about: Creates a rig attached to @mass and orientated relative to @con.
	@angle is how much the images orientation is offset.
	If @con is #Null then the image is orientated only by @angle.
	End Rem
	Function Create:TRig( image:TImage, mass:TMass, con:TConstraint=Null, angle:Float=0 )
		Local r:TRig=New TRig
		r.name="Rig"
		r.image=image
		r.mass=mass
		r.con=con
		r.angle=angle
		
		Return r
	End Function
	
	Rem
	bbdoc: Sets rigs name.
	End Rem
	Method SetName( n:String )
		name=n
	End Method
	
	Rem
	bbdoc: Gets rigs name.
	End Rem
	Method GetName:String()
		Return name
	End Method
	
	Rem
	bbdoc: Sets the TRigs image
	End Rem
	Method SetImage( img:TImage )
		image=img
	End Method
	
	Rem
	bbdoc: Gets the image currently associated with the #TRig.
	returns: The rigs current image.
	about: GetImage gives you access to the rigs image.
	End Rem
	Method GetImage:TImage()
		Return image
	End Method
	
	Rem
	bbdoc: Sets the TRigs mass.
	adout: The mass associated with the #TRig acts as it's handle for positioning.
	End Rem
	Method SetMass( m:TMass )
		mass=m
	End Method
	
	Rem
	bbdoc: Gets the mass associated with the #TRig.
	returns: The #TMass object that acts as the rigs handle.
	about: GetMass gives you access to the rigs #Tmass object.
	End Rem
	Method GetMass:TMass()
		Return mass
	End Method
	
	Rem
	bbdoc: Sets the TRigs constraint.
	about: The constraints orientation determines the rigs rotation.
	End Rem
	Method SetConstraint( c:TConstraint=Null )
		con=c
	End Method
	
	Rem
	bbdoc: Returns the #Tconstraint associated with the #TRig.
	returns: The #TConstraint of the rig.
	about: GetConstraint gives you access to the #TConstraint of the rig.
	If the rig has no #TConstraint associated with it then GetConstraint returns #null.
	End Rem
	Method GetConstraint:TConstraint()
		Return con
	End Method
	
	Rem
	bbdoc: Sets the TRigs angle offset.
	about: This sets the angular offset from the constraint at which the rig will be drawn.
	End Rem
	Method SetAngle( a:Float )
		angle=a
	End Method
	
	Rem
	bbdoc: Gets the rigs angular offset.
	returns: The difference between the rigs constraints angle and the angle at which the rigs image will be drawn.
	about: If the rig has no constraint associated with it GetAngle returns the angle at which the rigs image is drawn.
	End Rem
	Method GetAngle:Float()
		Return angle
	End Method
	
	Rem
	bbdoc: Sets the scale at which the rig is drawn.
	about: Sets the x and y scale of the rigs image.
	End Rem
	Method SetScale( valueX:Float, valueY:Float )
		scaleX=valueX
		scaleY=valueY
	End Method
	
	Rem
	bbdoc: Gets the rigs scale.
	about: Sets x and y to the rigs current x and y scale.
	End Rem
	Method GetScale( x:Float Var, y:Float Var )
		x=scaleX;y=scaleY
	End Method
	
	Rem
	bbdoc: Sets the tint of the rig.
	about: SetColor sets the color tint that the rig will be drawn with.
	End Rem
	Method SetColor( r:Int, g:Int, b:Int )
		red=Max(0,Min(255,r))/255.0
		green=Max(0,Min(255,g))/255.0
		blue=Max(0,Min(255,b))/255.0
	End Method
	
	Rem
	bbdoc: Gets the rigs color.
	about: Sets r, g and b to the red, green and blue values respectively of the rig.
	End Rem
	Method GetColor( r:Int Var, g:Int Var, b:Int Var )
		r=red*255;g=green*255;b=blue*255
	End Method
	
	Rem
	bbdoc: Sets the alpha value of the rig.
	about: The rigs alpha will only have an effect if the rigs blend mode is @ALPHABLEND or @LIGHTBLEND.
	End Rem
	Method SetAlpha( value:Float )
		alpha=Max(0,Min(1.0,value))
	End Method
	
	Rem
	bbdoc: Gets the rigs alpha value.
	returns: The alpha value of the rig.
	End Rem
	Method GetAlpha:Float()
		Return alpha
	End Method
	
	Rem
	bbdoc: Sets the rigs blend mode.
	about: Value should be one of @MASKBLEND, @SOLIDBLEND, @ALPHABLEND, @LIGHTBLEND or @SHADEBLEND.
	End Rem
	Method SetBlend( value:Int )
		blend=value
	End Method
	
	Rem
	bbdoc: Gets the rigs current blend mode.
	returns: The rigs current blend mode.
	about: The returned value will be one of @MASKBLEND, @SOLIDBLEND, @ALPHABLEND, @LIGHTBLEND or @SHADEBLEND.
	End Rem
	Method GetBlend:Int()
		Return blend
	End Method
	
	Rem
	bbdoc: Gets the index of the rig
	returns: The index of the rig
	about: GetIndex returns -1 if the rig isn't associated with a TAdvBody
	End Rem
	Method GetIndex:Int()
		Return index
	End Method
	
	Rem
	bbdoc: Gets the #TAdvBody that the rig is associated with.
	returns: The #TAdvBody of the rig.
	about: GetAdvBody returns #null if the rig isn't associated with an TAdvBody.
	End Rem
	Method GetAdvBody:TAdvBody()
		If abID<0 Or abID>=TAdvBody.aNext
			Return Null
		Else
			Return TAdvBody.array[abID]
		EndIf
	End Method
	
	Rem
	bbdoc: Draws the TRig
	about: Draws the TRig to the screen.
	The position and orientation are based on the mass and constraint associated with the TRig.
	The drawing of TRigs is scaled by using #PhysicsSetDrawScale.
	End Rem
	Method Draw()
		Local bm:Int=.GetBlend()
		Local a:Float=GetRotation()
		Local r:Int,g:Int,b:Int
		.GetColor r,g,b
		Local al:Float=.GetAlpha()
		Local sx:Float,sy:Float
		.GetScale sx,sy
		
		.SetBlend blend
		If con
			SetRotation con.GetAngle()+angle
		Else
			SetRotation angle
		EndIf
		.SetColor r*red,g*green,b*blue
		.SetAlpha al*alpha
		.SetScale scaleX,scaleY
		TPhysics.DrawImage image,mass.posX,mass.posY
		
		.SetBlend bm
		.SetColor r,g,b
		.SetAlpha al
		.SetScale sx,sy
		SetRotation a
	End Method
End Type

Rem
bbdoc: Collision zone object.
End Rem
Type TZone
	Const LINE:Int=0
	Const LOOP:Int=1
	Const FILL:Int=2
	Const HOLE:Int=3
	
	Global array:TZone[]
	Global aNext:Int=0
	Global stack:Int[]
	Global sNext:Int=0
	
	
	Field name:String
	
	Field origArea:Float[]
	Field area:Float[]
	Field style:Int
	Field friction:Float
	Field offsetX:Float, offsetY:Float, angle:Float=0.0, scaleX:Float=1.0, scaleY:Float=1.0
	Field red:Float=1.0, green:Float=1.0, blue:Float=1.0, alpha:Float=1.0
	Field blend:Int=MASKBLEND

	Field colGroup:Int=-1
	Field ColResponse( zone:TZone, mass:TMass, penNormX:Float, penNormY:Float, penMag:Float )=ColDefault
	Field quadTree:TQuadTree
	Field qtLevel:Int
	
	Field image:TImage
	Field imgScaleX:Float=1.0, imgScaleY:Float=1.0
	
	Field aIndex:Int=-1
	
	Rem
	bbdoc: Creates a new collision zone.
	returns: Collision zone created.
	about: A collision zone is an area of the simulation that masses can collide with.
	The collision @area is difined by an array of x:Float,y:Float coordinate pairs.
	@Style can be either @LINE, @LOOP, @FILL, or @HOLE.
	@colGroup relates to which masses will collide with the zone. See #TMass.Create for more information.
	@qtLevel is the number of levels in the zones QuadTree.
	A value of 1 is the same as having bounding checks and is sufficient in most cases.
	QuadTree creation is slow so setting qtLevel too high can cause massive delays in an application.
	End Rem
	Function Create:TZone( area:Float[], style:Int=LINE, frict:Float=1.0, colGroup:Int=-1, qtLevel:Int=1 )
		If area.length<4 Or (area.length&1) Return Null
		
		Local z:TZone=New TZone
		z.name="Zone"
		z.origArea=area
		z.area=area[..]
		z.style=style
		z.friction=Max(0,Min(1,frict))
		
		If colGroup=-1
			z.colGroup=TPhysics.cgCounter
			TPhysics.cgCounter:-1
		Else
			z.colGroup=colGroup
		EndIf
		
		z.Activate
		
		z.CalcQuadTree qtLevel
		Return z
	End Function
	
	Rem
	bbdoc: Loads a saved collision zone.
	returns: Loaded collision zone.
	about: @url is the file where the TZone is saved.
	The zones QuadTree is also loaded so does not need to be recalculated.
	End Rem
	Function Load:TZone( url:Object, active:Int=True )
		'setup file ready for reading
		Local file:TStream=ReadStream(url)
		If Not file Then Return Null
		file=LittleEndianStream(file)
		
		Local header:String=ReadString(file,4)
		Local version:Int=ReadByte(file)
		If header<>"ZONE"
			CloseStream file
			Return Null
		EndIf
		
		Local z:TZone=New TZone
		
		z.name=ReadLine(file)
		
		Local aLength:Int=ReadInt(file)
		z.area=New Float[aLength]
		z.origArea=New Float[aLength]
		For Local count:Int=0 To aLength-1
			z.origArea[count]=ReadFloat(file)
		Next
		
		z.style=ReadInt(file)
		z.friction=ReadFloat(file)
		z.offsetX=ReadFloat(file)
		z.offsetY=ReadFloat(file)
		z.angle=ReadFloat(file)
		z.scaleX=ReadFloat(file)
		z.scaleY=ReadFloat(file)
		z.red=ReadFloat(file)
		z.green=ReadFloat(file)
		z.blue=ReadFloat(file)
		z.alpha=ReadFloat(file)
		z.blend=ReadInt(file)
		z.SetColGroup ReadInt(file)
		z.qtLevel=ReadInt(file)
		z.imgScaleX=ReadFloat(file)
		z.imgScaleY=ReadFloat(file)
		
		z.quadTree=TQuadTree.Read(file)
		
		Local img:Int=ReadByte(file)
		If img
			Local mr:Int=ReadByte(file)
			Local mg:Int=ReadByte(file)
			Local mb:Int=ReadByte(file)
			SetMaskColor mr,mg,mb
			Local flags:Int=ReadByte(file)
			Local hx:Float=ReadFloat(file)
			Local hy:Float=ReadFloat(file)
			Local packed:Int=ReadByte(file)
			If packed
				z.image=LoadImage(file,flags)
			Else
				z.image=LoadImage(StripExt(RealPath(String(url)))+"_img.png",flags)
			EndIf
			SetImageHandle z.image,hx,hy
		EndIf
		
		CloseStream file
		
		z.UpdateArea
		z.Activate
		
		Return z
	
	End Function
	
	Rem
	bbdoc: Counts the number of active TZones.
	returns: The number of active TZones.
	about: CountActive only counts TZones that have been activated.
	End Rem
	Function CountActive:Int()
		Return aNext-sNext
	End Function
	
	Rem
	bbdoc: Maps one function to all active TZones.
	about: ApplyFunc is the function that will be run for every active TZone in the simulation.
	End Rem
	Function ApplyToActive( ApplyFunc( zone:TZone ) )
		For Local count:Int=0 Until aNext
			If array[count]
				ApplyFunc array[count]
			EndIf 
		Next
	End Function
	
	Rem
	bbdoc: Saves a collision zone.
	about: Saves a zone to a designated by @url.
	If no file extention is given then ".plz" is appended to the end of @url.
	
	If backup is set to true the a backup of any exist file with the same path will be made.
	
	Setting packImage to true will save the zones image in the .plz file.
	If it is set to false it is saved along side the zones file with the same file name, but prefixed "_img.png".
	
	Save does not save custom collision responses that have been set for the zone using #OverrideCollision.
	These can only be added at runtime.
	End Rem
	Method Save( url:Object, backup:Int=True, packImage:Int=True )
		Local file:TStream=TStream(url)
		Local str_url:String=String(url)
		If file
			file=WriteStream(file)
			packImage=True
		ElseIf str_url
			str_url=RealPath(str_url)
			Select FileType(str_url)
			Case 2
				Return
			Case 1
				If backup Then CopyFile str_url,str_url+".bak"
				DeleteFile str_url
			End Select
			If str_url=StripExt(str_url) Then str_url=str_url+".plz"
			file=WriteStream(str_url)
		Else
			Return
		EndIf
		
		file=LittleEndianStream(file)
		
		WriteString file,"ZONE"
		WriteByte file,112
		WriteLine file,GetName()
		
		WriteInt file,origArea.length
		For Local count:Int=0 To origArea.length-1
			WriteFloat file,origArea[count]
		Next
		
		WriteInt file,style
		WriteFloat file,friction
		WriteFloat file,offsetX
		WriteFloat file,offsetY
		WriteFloat file,angle
		WriteFloat file,scaleX
		WriteFloat file,scaleY
		WriteFloat file,red
		WriteFloat file,green
		WriteFloat file,blue
		WriteFloat file,alpha
		WriteInt file,blend
		WriteInt file,Max(colGroup,-1)
		WriteInt file,qtLevel
		WriteFloat file,imgScaleX
		WriteFloat file,imgScaleY
		
		quadTree.Write File
		
		If image
			WriteByte file,True
			WriteByte file,image.mask_r
			WriteByte file,image.mask_g
			WriteByte file,image.mask_b
			WriteByte file,image.flags
			WriteFloat file,image.handle_x
			WriteFloat file,image.handle_y
			WriteByte file,packImage
			If packImage
				SavePixmapPNG image.pixmaps[0],file,9
			Else
				SavePixmapPNG image.pixmaps[0],StripExt(str_url)+"_img.png"
			EndIf
		Else
			WriteByte file,False
		EndIf
		
		CloseStream file
		
	End Method
	
	Rem
	bbdoc: Cretaes a copy of the zone.
	returns: The zones copy.
	about: The new zone will be identical to the original.
	The new zone will initially share the originals image and quadtree.
	However any subsiquent changes to the originals quadtree or image will @not affect the copy.
	End Rem
	Method Copy:TZone()
		Local z:TZone=New TZone
		
		z.name=name
		z.origArea=origArea[..]
		z.area=area[..]
		z.style=style
		z.friction=friction
		z.offsetX=offsetX
		z.offsetY=offsetY
		z.angle=angle
		z.scaleX=scaleX
		z.scaleY=scaleY
		z.Red=red
		z.green=green
		z.blue=blue
		z.alpha=alpha
		z.blend=blend
		z.colGroup=colGroup
		z.qtLevel=qtLevel
		z.imgScaleX=imgScaleX
		z.imgScaleY=imgScaleY
		z.quadTree=quadTree
		z.image=image
		z.colResponse=colResponse
		
		z.Activate
		
		Return z
	End Method
	
	Rem
	bbdoc: Makes the collision zone active in the simulation.
	about: Activates a de-active collision zone.
	End Rem
	Method Activate()
		If aIndex=-1
			If sNext>0
				sNext:-1
				aIndex=stack[sNext]
			Else
				aIndex=aNext
				aNext:+1
				If aNext=array.length
					array=array[..aNext*2]
					stack=stack[..aNext*2]
				EndIf
			EndIf
			array[aIndex]=Self
		EndIf
	End Method
	
	Rem
	bbdoc: De-activates collision zone.
	about: A de-active collision zone does not create collision events or responses.
	End Rem
	Method DeActivate()
		If aIndex>-1
			stack[sNext]=aIndex
			sNext:+1
			array[aIndex]=Null
			aIndex=-1
			If aNext=sNext
				aNext=0
				sNext=0
			EndIf
		EndIf
	End Method
	
	Rem
	bbdoc: Removes TZone from the simulation.
	End Rem
	Method Free()
		DeActivate
	End Method
	
	Rem
	bbdoc: Sets the zones name.
	End Rem
	Method SetName( n:String )
		name=n
	End Method
	
	Rem
	bbdoc: Gets the zones name.
	returns: The zones name.
	End Rem
	Method GetName:String()
		Return name
	End Method
	
	Rem
	bbdoc: Sets collision zones friction.
	about: Friction should always be positive.
	End Rem
	Method SetFriction( f:Float )
		friction=Max(0,Min(1,f))
	End Method
	
	Rem
	bbdoc: Gets the zones friction value.
	returns: The zones friction.
	End Rem
	Method GetFriction:Float()
		Return friction
	End Method
	
	Rem
	bbdoc: Sets the zones style.
	about: A zones style determines how #TMass objects collide with it.
	@Value can be either @LINE, @LOOP, @FILL, or @HOLE.
	
	[ @LINE | The zone is a line that %isn't joined at both end
	* @LOOP | The zone is a line that %is joined at each end to form a loop
	* @FILL | The zone is a solid polygon
	* @HOLE | The @HOLE style is the inverse of @FILL
	]
	End Rem
	Method SetStyle( value:Int )
		style=value
	End Method
	
	Rem
	bbdoc: Gets the zones current style.
	returns: The zones current style.
	about: GetStyle returns either @LINE, @LOOP, @FILL, or @HOLE.
	End Rem
	Method GetStyle:Int()
		Return style
	End Method
	
	Rem
	bbdoc: Positions the collision zone within the simulation.
	End Rem
	Method Position( x:Float, y:Float )
		offsetX=x
		offsetY=y
	End Method
	
	Rem
	bbdoc: Moves the collision zone relative to it's current position.
	about: TZones are intended to be stationary, and it is recomended not to move a TZone if it may collide with active masses.
	If a dynamic collision area is needed consider using a TBody instead.
	End Rem
	Method Move( x:Float, y:Float )
		offsetX:+x
		offsetY:+y
	End Method
	
	Rem
	bbdoc: Gets the current X position of the TZone.
	End Rem
	Method GetPositionX:Float()
		Return offsetX
	End Method
	
	Rem
	bbdoc: Gets the current Y position of the TZone.
	End Rem
	Method GetPositionY:Float()
		Return offsetY
	End Method
	
	Rem
	bbdoc: Gets the current X position of the TZone.
	End Rem
	Method GetX:Float()
		Return offsetX
	End Method
	
	Rem
	bbdoc: Gets the current Y position of the TZone.
	End Rem
	Method GetY:Float()
		Return offsetY
	End Method
	
	Rem
	bbdoc: Rotates collision zone
	about: Not recommended for realtime use.
	End Rem
	Method Rotate( value:Float )
		angle=value
		UpdateArea
		
		CalcQuadTree qtLevel
	End Method
	
	Rem
	bbdoc: Turns collision zone
	about: Not recommended for realtime use.
	End Rem
	Method Turn( value:Float )
		angle:+value
		UpdateArea
		
		CalcQuadTree qtLevel
	End Method
	
	Rem
	bbdoc: Gets the current rotation of the zone.
	returns: The zones rotation.
	End Rem
	Method GetRotation:Float()
		Return angle
	End Method
	
	Rem
	bbdoc: Sets the zones scale.
	about: SetZone scales the collision zone by x and y.
	SetScale will also affect the zones image scale.
	End Rem
	Method SetScale( x:Float, y:Float )
		scaleX=x
		scaleY=y
		UpdateArea
		
		CalcQuadTree qtLevel
	End Method
	
	Rem
	bbdoc: Gets the zones scale.
	about: Sets x and y to the zones current scale.
	End Rem
	Method GetScale( x:Float Var, y:Float Var )
		x=scaleX
		y=scaleY
		UpdateArea
		
		CalcQuadTree qtLevel
	End Method
	
	Rem
	bbdoc: Sets the zones color.
	about: If no image is set for the zone then this will be the zones color.
	Otherwise SetColor will tint the images color.
	End Rem
	Method SetColor( r:Int, g:Int, b:Int )
		red=Max(0,Min(255,r))/255.0
		green=Max(0,Min(255,g))/255.0
		blue=Max(0,Min(255,b))/255.0
	End Method
	
	Rem
	bbdoc: Gets the zones current color setting.
	about: Sets r, g and b to the zones current red, green and blue value respectively.
	End Rem
	Method GetColor( r:Int Var, g:Int Var, b:Int Var )
		r=red*255;g=green*255;b=blue*255
	End Method
	
	Rem
	bbdoc: Sets the zones alpha.
	about: SetAlpha will only have an effect on zones with the alpha or light blend mode.
	End Rem
	Method SetAlpha( value:Float )
		alpha=value
	End Method
	
	Rem
	bbdoc: Gets the zones current alpha.
	returns: The zones alpha.
	End Rem
	Method GetAlpha:Float()
		Return alpha
	End Method
	
	Rem
	bbdoc: Sets the zones blend mode.
	about: Value can be one of @MASKBLEND, @SOLIDBLEND, ALPHABLEND, LIGHTBLEND or SHADEBLEND.
	End Rem
	Method SetBlend( value:Int )
		blend=value
	End Method
	
	Rem
	bbdoc: Gets the zones current blend mode.
	returns: The zones blend mode.
	End Rem
	Method GetBlend:Int()
		Return blend
	End Method
	
	Rem
	bbdoc: Counts the number of vertex in the zones area.
	returns: The number of vertex in the zones area.
	End Rem
	Method VertCount:Int()
		Return (origArea.length+1)/2
	End Method
	
	Rem
	bbdoc: Adds a vertex to the zones area.
	about: Adds a vertex to the end of the zones area at position x,y.
	End Rem
	Method VertAddLast( x:Float, y:Float )
		Local length:Int=origArea.length+2
		Local pos:Int=origArea.length
		origArea=origArea[..length]
		area=area[..length]
		origArea[pos]=x
		origArea[pos+1]=y
		x:*scaleX
		y:*scaleY
		Local mag:Float=Sqr(x*x+y*y)
		Local ang:Float=ATan2(y,x)+angle
		area[pos]=Cos(ang)*mag
		area[pos+1]=Sin(ang)*mag
		
		CalcQuadTree qtLevel
	End Method
	
	Rem
	bbdoc: Adds a vertex to the zones area.
	about: Adds a vertex to the beginning of the zones area at position x,y.
	End Rem
	Method VertAddFirst( x:Float, y:Float )
		origArea=origArea[-2..]
		area=area[-2..]
		origArea[0]=x
		origArea[1]=y
		x:*scaleX
		y:*scaleY
		Local mag:Float=Sqr(x*x+y*y)
		Local ang:Float=ATan2(y,x)+angle
		area[0]=Cos(ang)*mag
		area[1]=Sin(ang)*mag
		
		CalcQuadTree qtLevel
	End Method
	
	Rem
	bbdoc: Adds a vertex to a zones area.
	about: Inserts a vertex into a zones area at index and at position x,y.
	End Rem
	Method VertInsert( index:Int, x:Float, y:Float )
		If index>VertCount() Or index<0 Then Return
		Local length:Int=origArea.length+2
		Local pos:Int=index*2
		origArea=origArea[..length]
		area=area[..length]
		For Local count:Int=length-1 To pos+2 Step -1
			origArea[count]=origArea[count-2]
			area[count]=area[count-2]
		Next
		origArea[pos]=x
		origArea[pos+1]=y
		x:*scaleX
		y:*scaleY
		Local mag:Float=Sqr(x*x+y*y)
		Local ang:Float=ATan2(y,x)+angle
		area[pos]=Cos(ang)*mag
		area[pos+1]=Sin(ang)*mag
		
		CalcQuadTree qtLevel
	End Method
	
	Rem
	bbdoc: Removes a vertex from the zones area.
	about: Removes the last vertex from a zone.
	End Rem
	Method VertRemoveLast()
		If VertCount()<3 Then Return
		origArea=origArea[..origArea.length-2]
		area=area[..area.length-2]
		
		CalcQuadTree qtLevel
	End Method
	
	Rem
	bbdoc: Removes a vertex from the zones area.
	about: Removes the last vertex from a zone area.
	End Rem
	Method VertRemoveFirst()
		If VertCount()<3 Then Return
		origArea=origArea[2..]
		area=area[2..]
		
		CalcQuadTree qtLevel
	End Method
	
	Rem
	bbdoc: Removes a vertex from a zones area.
	about: Removes vertex @index from the zones area.
	End Rem
	Method VertRemove( index:Int )
		If VertCount()<3 Or index>=VertCount() Or index<0 Then Return
		index:*2
		origArea=origArea[..index]+origArea[index+2..]
		Area=Area[..index]+Area[index+2..]
		
		CalcQuadTree qtLevel
	End Method
	
	Rem
	bbdoc: Positions a vertex.
	about: Moves vertex @index to position x,y.
	VertPosition uses the zones local coordinate system.
	This means x,y is the position if the zone isn't rotated, positioned or scaled.
	End Rem
	Method VertPosition( index:Int, x:Float, y:Float )
		If index>=VertCount() Or index<0 Then Return
		index:*2
		origArea[index]=x
		origArea[index+1]=y
		x:*scaleX
		y:*scaleY
		Local mag:Float=Sqr(x*x+y*y)
		Local ang:Float=ATan2(y,x)+angle
		area[index]=Cos(ang)*mag
		area[index+1]=Sin(ang)*mag
		
		CalcQuadTree qtLevel
	End Method
	
	Rem
	bbdoc: Gets a vertex's position.
	about: Sets x and y to the vertex's position.
	The values area given in the zones local coordinate system.
	i.e. As if the zone wasn't rotated, positioned or scaled.
	End Rem
	Method VertGetPosition( index:Int, x:Float Var, y:Float Var )
		If index>=VertCount() Or index<0 Then Return
		index:*2
		x=origArea[index]
		y=origArea[index+1]
	End Method
	
	Rem
	bbdoc: Gets the vertex's x coordinate.
	returns: The vertex's x coordinate.
	End Rem
	Method VertGetX:Float( index:Int )
		index:*2
		Return origArea[index]
	End Method
	
	Rem
	bbdoc: Gets the vertex's y coordinate.
	returns: The vertex's y coordinate.
	End Rem
	Method VertGetY:Float( index:Int )
		index:*2
		Return origArea[index+1]
	End Method
	
	Rem
	bbdoc: Recalculates zones QuadTree.
	End Rem
	Method CalcQuadTree( level:Int=1 )
		qtlevel=level
		Local x:Float,y:Float,maxX:Float,maxY:Float
		x=area[0]
		y=area[1]
		maxX=area[0]
		maxY=area[1]
		For Local count:Int=2 Until area.Length Step 2
			x=Min(x,area[count])
			y=Min(y,area[count+1])
			maxX=Max(maxX,area[count])
			maxY=Max(maxY,area[count+1])
		Next
		quadTree=TQuadTree.Create(area,style,x-1,y-1,maxX-x+2,maxY-y+2,qtlevel)
	End Method
	
	Rem
	bbdoc: Gets the zones current quadtree level.
	returns: The zones quadtree level.
	End Rem
	Method GetQTLevel:Int()
		Return qtLevel
	End Method
	
	Rem
	bbdoc: Gets the zones area.
	returns: Collision zone.
	about: #GetArea returns an array of x:Float, y:Float coordinate pairs.
	The coordinates are rotated, but are @not offset by the zones current position.
	End Rem
	Method GetArea:Float[]()
		Return area[..]
	End Method
	
	Rem
	bbdoc: Sets the zones collision group
	about: For more info on collision groups see #TMass.SetColGroup
	End Rem
	Method SetColGroup( value:Int=-1 )
		If value=-1
			colGroup=TPhysics.cgCounter
			TPhysics.cgCounter:-1
		Else
			colGroup=value
		EndIf
	End Method
	
	Rem
	bbdoc: Gets the collision group of the TZone
	returns: Zones collision group
	End Rem
	Method GetColGroup:Int()
		Return colGroup
	End Method
	
	Rem
	bbdocs: Sets the TZone image.
	about: Sets the image used by the TZone's #Draw method.
	EndRem
	Method ImageSet( img:TImage )
		image=img
	End Method
	
	Rem
	bbdoc: Gets the zones current image.
	returns: The zones current image.
	about: ImageGet returns #null if no image is associated with the zone.
	End Rem
	Method ImageGet:TImage()
		Return image
	End Method
	
	Rem
	bbdoc: Sets the zones image scale.
	about: Sets the scale that a zones image will be drawn with.
	
	%Note: SetScale also affects the scale that a zones image is drawn with.
	End Rem
	Method ImageSetScale( x:Float, y:Float )
		imgScaleX=x
		imgScaleY=y
	End Method
	
	Rem
	bbdoc: Gets the zones current image scale.
	about: Sets x and y to the zones images current scale.
	End Rem
	Method ImageGetScale( x:Float Var, y:Float Var )
		x=imgScaleX
		y=imgScaleY
	End Method
	
	Rem
	bbdoc: Draws the TZone
	about: Draws the TZone to the display.
	If an image has not been set for the TZone then a polygon is drawn.
	Draw is affected by #PhysicsSetDrawScale.
	End Rem
	Method Draw()
		Local bm:Int=.GetBlend()
		Local a:Float=.GetRotation()
		Local r:Int,g:Int,b:Int
		.GetColor r,g,b
		Local al:Float=.GetAlpha()
		Local sx:Float,sy:Float
		.GetScale sx,sy
		
		.SetBlend blend
		.SetColor r*red,g*green,b*blue
		.SetAlpha al*alpha
		If image
			.SetRotation angle
			.SetScale scaleX*imgScaleX,scaleY*imgScaleY
			TPhysics.DrawImage image,offsetX,offsetY
		Else
			.SetRotation 0
			.SetScale 1.0,1.0
			Select style
			Case LINE
				For Local count:Int=0 To area.length-4 Step 2
					PhysicsDrawLine area[count]+offsetX,area[count+1]+offsetY,area[count+2]+offsetX,area[count+3]+offsetY
				Next
			Case LOOP,HOLE
				For Local count:Int=0 To area.length-4 Step 2
					PhysicsDrawLine area[count]+offsetX,area[count+1]+offsetY,area[count+2]+offsetX,area[count+3]+offsetY
				Next
				PhysicsDrawLine area[area.length-2]+offsetX,area[area.length-1]+offsetY,area[0]+offsetX,area[1]+offsetY
			Case FILL
				TPhysics.DrawPoly area,offsetX,offsetY
			End Select
		EndIf
		
		.SetBlend bm
		.SetColor r,g,b
		.SetAlpha al
		.SetScale sx,sy
		.SetRotation a
	End Method
	
	Rem
	bbdoc: Draws the images quadtree.
	about: DrawQuadTree is too slow for serious realtime use, but can be an invaluable debugging tool.
	End Rem
	Method DrawQuadTree( filled:Int=True )
		quadTree.Draw offsetX,offsetY,filled
	End Method
	
	Rem
	bbdoc: Sets the function to be used for collisions.
	about: If no function is named the default collision function will be used.
	Recommended for advanced users only.
	End Rem
	Method OverrideCollision( CustomResponse(zone:TZone,mass:TMass,penNormX:Float,penNormY:Float,penMag:Float)=Null )
		
		If CustomResponse Then ColResponse=CustomResponse Else ColResponse=ColDefault
		
	End Method
	
	'WARNING for internal use only
	Method UpdateArea()
		For Local count:Int=0 To area.Length-1 Step 2
			Local x:Float=origArea[count]*scaleX
			Local y:Float=origArea[count+1]*scaleY
			Local mag:Float=Sqr(x*x+y*y)
			Local ang:Float=ATan2(y,x)+angle
			area[count]=Cos(ang)*mag
			area[count+1]=Sin(ang)*mag
		Next
	End Method
	
	'WARNING for internal use only
	Method Collide( m:TMass )
		Local colX:Float
		Local colY:Float
		Local penVectX:Float
		Local penVectY:Float
		Local penNormX:Float
		Local penNormY:Float
		Local penMag:Float
		Local dist:Float=PHYS_MAX_FLOAT
		
		If Not quadTree.Collide(offsetX,offsetY,m.posX-m.radius,m.posY-m.radius,m.radius*2,style&HOLE) Then Return
		
		Select style
		Case LINE
			Local x1:Float=area[0]+offsetX
			Local y1:Float=area[1]+offsetY
			For Local i:Int=2 Until Len area Step 2
				Local x2:Float=area[i]+offsetX
				Local y2:Float=area[i+1]+offsetY
				
				Local sx:Float = x2-x1
				Local sy:Float = y2-y1
				
				Local q:Float = ((m.posX-x1) * (x2-x1) + (m.posY - y1) * (y2-y1)) / (sx*sx + sy*sy)
				
				If q < 0.0 Then q = 0.0
				If q > 1.0 Then q = 1.0
				
				Local cx:Float=(1-q)*x1+q*x2
				Local cy:Float=(1-q)*y1 + q*y2
				
				Local tmp:Float=(m.posX-cx)*(m.posX-cx)+(m.posY-cy)*(m.posY-cy)
				
				If tmp<dist
					dist=tmp
					colX=cx
					colY=cy
				EndIf
				
				x1=x2
				y1=y2
				
			Next
			
			penVectX=m.posX-colX
			penVectY=m.posY-colY
			dist=Sqr(dist)
			If dist<PHYS_MIN_FLOAT Then dist=PHYS_MIN_FLOAT
			penNormX=penVectX/dist
			penNormY=penVectY/dist
			penMag=m.radius-dist
			
		Case LOOP
			
			Local x1:Float=area[area.Length-2]+offsetX
			Local y1:Float=area[area.Length-1]+offsetY
			For Local i:Int=0 Until Len area Step 2
				Local x2:Float=area[i]+offsetX
				Local y2:Float=area[i+1]+offsetY
				
				Local sx:Float = x2-x1
				Local sy:Float = y2-y1
				
				Local q:Float = ((m.posX-x1) * (x2-x1) + (m.posY - y1) * (y2-y1)) / (sx*sx + sy*sy)
				
				If q < 0.0 Then q = 0.0
				If q > 1.0 Then q = 1.0
				
				Local cx:Float=(1-q)*x1+q*x2
				Local cy:Float=(1-q)*y1 + q*y2
				
				Local tmp:Float=(m.posX-cx)*(m.posX-cx)+(m.posY-cy)*(m.posY-cy)
				
				If tmp<dist
					dist=tmp
					colX=cx
					colY=cy
				EndIf
				
				x1=x2
				y1=y2
				
			Next
			
			penVectX=m.posX-colX
			penVectY=m.posY-colY
			dist=Sqr(dist)
			If dist<PHYS_MIN_FLOAT Then dist=PHYS_MIN_FLOAT
			penNormX=penVectX/dist
			penNormY=penVectY/dist
			penMag=m.radius-dist
			
		Case FILL
			
			Local x1:Float=area[area.Length-2]+offsetX
			Local y1:Float=area[area.Length-1]+offsetY
			Local cur_quad:Int
			If x1<m.posX
				If y1<m.posY
					cur_quad=1
				Else
					cur_quad=4
				EndIf
			Else
				If y1<m.posY
					cur_quad=2
				Else
					cur_quad=3
				EndIf	
			EndIf
			Local total:Int
			
			For Local i:Int=0 Until Len area Step 2
				Local x2:Float=area[i]+offsetX
				Local y2:Float=area[i+1]+offsetY
				Local next_quad:Int
				If x2<m.posX
					If y2<m.posY
						next_quad=1
					Else
						next_quad=4
					EndIf
				Else
					If y2<m.posY
						next_quad=2
					Else
						next_quad=3
					EndIf	
				EndIf
				Local diff:Int=next_quad-cur_quad
				
				Select diff
				Case 2,-2
					If ( x2 - ( ((y2 - m.posY) * (x1 - x2)) / (y1 - y2) ) )<m.posX
						diff=-diff
					EndIf
				Case 3
					diff=-1
				Case -3
					diff=1
				End Select
				
				total:+diff
				cur_quad=next_quad
				
				Local sx:Float = x2-x1
				Local sy:Float = y2-y1
				
				Local q:Float = ((m.posX-x1) * (x2-x1) + (m.posY - y1) * (y2-y1)) / (sx*sx + sy*sy)
				
				If q < 0.0 Then q = 0.0
				If q > 1.0 Then q = 1.0
				
				Local cx:Float=(1-q)*x1+q*x2
				Local cy:Float=(1-q)*y1 + q*y2
				
				Local tmp:Float=(m.posX-cx)*(m.posX-cx)+(m.posY-cy)*(m.posY-cy)
				
				If tmp<dist
					dist=tmp
					colX=cx
					colY=cy
				EndIf
				
				x1=x2
				y1=y2
			Next
			
			dist=Sqr(dist)
			If dist<PHYS_MIN_FLOAT Then dist=PHYS_MIN_FLOAT
			
			If Abs(total)=4
				
				penVectX=colX-m.posX
				penVectY=colY-m.posY
				penNormX=penVectX/dist
				penNormY=penVectY/dist
				penMag=dist+m.radius
				
			Else
				
				penVectX=m.posX-colX
				penVectY=m.posY-colY
				penNormX=penVectX/dist
				penNormY=penVectY/dist
				penMag=m.radius-dist
				
			EndIf
			
		Case HOLE
			
			Local x1:Float=area[area.Length-2]+offsetX
			Local y1:Float=area[area.Length-1]+offsetY
			Local cur_quad:Int
			If x1<m.posX
				If y1<m.posY
					cur_quad=1
				Else
					cur_quad=4
				EndIf
			Else
				If y1<m.posY
					cur_quad=2
				Else
					cur_quad=3
				EndIf	
			EndIf
			Local total:Int
			
			For Local i:Int=0 Until Len area Step 2
				Local x2:Float=area[i]+offsetX
				Local y2:Float=area[i+1]+offsetY
				Local next_quad:Int
				If x2<m.posX
					If y2<m.posY
						next_quad=1
					Else
						next_quad=4
					EndIf
				Else
					If y2<m.posY
						next_quad=2
					Else
						next_quad=3
					EndIf	
				EndIf
				Local diff:Int=next_quad-cur_quad
				
				Select diff
				Case 2,-2
					If ( x2 - ( ((y2 - m.posY) * (x1 - x2)) / (y1 - y2) ) )<m.posX
						diff=-diff
					EndIf
				Case 3
					diff=-1
				Case -3
					diff=1
				End Select
				
				total:+diff
				cur_quad=next_quad
				
				Local sx:Float = x2-x1
				Local sy:Float = y2-y1
				
				Local q:Float = ((m.posX-x1) * (x2-x1) + (m.posY - y1) * (y2-y1)) / (sx*sx + sy*sy)
				
				If q < 0.0 Then q = 0.0
				If q > 1.0 Then q = 1.0
				
				Local cx:Float=(1-q)*x1+q*x2
				Local cy:Float=(1-q)*y1 + q*y2
				
				Local tmp:Float=(m.posX-cx)*(m.posX-cx)+(m.posY-cy)*(m.posY-cy)
				
				If tmp<dist
					dist=tmp
					colX=cx
					colY=cy
				EndIf
				
				x1=x2
				y1=y2
			Next
			
			dist=Sqr(dist)
			If dist<PHYS_MIN_FLOAT Then dist=PHYS_MIN_FLOAT
			
			If Abs(total)=4
				
				penVectX=m.posX-colX
				penVectY=m.posY-colY
				penNormX=penVectX/dist
				penNormY=penVectY/dist
				penMag=m.radius-dist
				
			Else
				
				penVectX=colX-m.posX
				penVectY=colY-m.posY
				penNormX=penVectX/dist
				penNormY=penVectY/dist
				penMag=dist+m.radius
				
			EndIf
			
		End Select
		
		If penMag>0
			ColResponse(Self,m,penNormX,penNormY,penMag)
			TColEvent.Emit TColEvent.Create(m,Self,colX,colY,penMag)
		EndIf
	End Method
	
	Function ColDefault( zone:TZone, mass:TMass, penNormX:Float, penNormY:Float, penMag:Float )
		
		Local penVectX:Float=penNormX*-penMag
		Local penVectY:Float=penNormY*-penMag
		Local frictCoef:Float=penMag*(zone.friction+mass.friction)*0.5
		
		mass.posX:-penVectX
		mass.posY:-penVectY
		
		'Friction
		Local velVectX:Float=mass.posX-mass.oldposX
		Local velVectY:Float=mass.posY-mass.oldPosY
		Local vel:Float=Sqr(velVectX*velVectX+velVectY*velVectY)
		Local angle:Float=ATan2(velVectY,velVectX)-ATan2(penNormY,penNormX)
		Local tanVel:Float=Cos(angle)*vel
		Local tanVectX:Float=penNormX*tanVel
		Local tanVectY:Float=penNormY*tanVel
		tanVectX=velVectX-tanVectX
		tanVectY=velVectY-tanVectY
		tanVel=Sqr(tanVectX*tanVectX+tanVectY*tanVectY)
'Rem				
		If frictCoef>=tanVel
			mass.PosX:-tanVectX
			mass.PosY:-tanVectY
		Else
			mass.PosX:-(tanVectX/tanVel)*frictCoef
			mass.PosY:-(tanVectY/tanVel)*frictCoef
		EndIf
Rem
		If frictCoef>=tanVel
			mass.oldPosX:+tanVectX
			mass.oldPosY:+tanVectY
		Else
			mass.oldPosX:+(tanVectX/tanVel)*frictCoef
			mass.oldPosY:+(tanVectY/tanVel)*frictCoef
		EndIf
EndRem
	End Function
End Type

Rem
bbdoc: Collision Event object.
End Rem
Type TColEvent
	Const MASS_CUSTOM:Int=-1
	Const MASS_MASS:Int=1
	Const MASS_ZONE:Int=2
	Const MASS_BOUNDS:Int=3
	
	Const MODE_DISCARD:Int=0
	Const MODE_UPDATE:Int=1
	Const MODE_REPLACE:Int=2
	
	Rem
	bbdoc: The collision event hook ID.
	about: The hook event which can be used by #AddHook to add a hook function.
	End Rem
	Global Hook:Int=AllocHookId()
	Global current:TColEvent
	Global queue:TList=New TList
	Global map:TMap=New TMap
	Global mode:Int=MODE_UPDATE
	
	Field id:Int
	Field obj1:TMass
	Field obj2:Object
	Field x:Float, y:Float
	Field force:Float
	
	Rem
	bbdoc: Creates a collision event object.
	returns: The created collision event.
	about: Craetes a collision event that can then be emitted or posted.
	End Rem
	Function Create:TColEvent( obj1:TMass, obj2:Object, x:Float, y:Float, force:Float )
		Local ce:TColEvent=New TColEvent
		
		If TMass(obj2)
			ce.id=MASS_MASS
		ElseIf TZone(obj2)
			ce.id=MASS_ZONE
		ElseIf TPhysics(obj2)
			ce.id=MASS_BOUNDS
		Else
			ce.id=MASS_CUSTOM
		EndIf
		ce.obj1=obj1
		ce.obj2=obj2
		ce.x=x;ce.y=y
		ce.force=force
		
		Return ce
	End Function
	
	Rem
	bbdoc: Sets how collision events are added to the event queue
	about: Sets the way collision events are added to the event queue.
	If there is more than one iteration then it is possible for two objects to collide more than once during the same update.
	#SetMode tells PhysLite how to report multiple collision between the same objects.
	@Value can be one of the following values, @MODE_DISCARD, @MODE_UPDATE or @MODE_REPLACE.
	@MODE_DISCARD means no events will be added to the event queue, although hooked functions will still be called.
	Setting @MODE_DISCARD will give a speed increase.
	It is also useful for making your own custom  event queue using hook functions.
	@MODE_UPDATE will total the force of multiple collisions between the same objects. This is the default mode.
	@MODE_REPLACE will discard previous collision events between the same two objects and only report the last.
	End Rem
	Function SetMode( value:Int )
		mode=value
	End Function
	
	Rem
	bbdoc: Emits a collision event
	about: When a collision event is emitted all hooked functions are run and then the event is posted to the event queue.
	If the hook functions return @Null then the collision event will not be posted to the event queue.
	Emitting is usually handled automatically by #TPhysics.Update but #Emit can be used to fake a collision event.
	End Rem
	Function Emit( colEvent:TColEvent )
		colEvent=TColEvent(RunHooks(hook,colEvent))
		If colEvent And mode Then Post colEvent,mode
	End Function
	
	Rem
	bbdoc: Posts a collision event to the collision events queue
	about: Similar to #Emit, #Post will add a collision event to the collision event queue.
	However posted collision events will not run any hooked functions.
	The @update parameter is used to tell PhysLite to either update or replace previous collisions between the same objects.
	See #SetMode for more information on modes.
	End Rem
	Function Post( colEvent:TColEvent, update:Int=MODE_UPDATE )
		Local obj1:Object
		Local obj2:Object
		If colEvent.obj1.Compare(colEvent.obj2)<0
			obj1=colEvent.obj1
			obj2=colEvent.obj2
		Else
			obj1=colEvent.obj2
			obj2=colEvent.obj1
		EndIf
		Local map2:TMap=TMap(map.ValueForKey(obj1))
		If map2
			Local ce:TColEvent=TColEvent(map2.ValueForKey(obj2))
			If ce
				ce.x=colEvent.x
				ce.y=colEvent.y
				If mode=MODE_REPLACE
					ce.force=colEvent.force
				Else
					ce.force:+colEvent.force
				EndIf
			Else
				map2.Insert obj2,colEvent
				queue.AddLast colEvent
			EndIf
		Else
			map2=CreateMap()'New TMap
			map2.Insert obj2,colEvent
			map.Insert obj1,map2
			queue.AddLast colEvent
		EndIf
	End Function
	
	Rem
	bbdoc: Removes the next collision event from the queue.
	returns: The next collision event in the queue.
	about: #Poll sequentially removes collision events from the queue, and sets it to the current event.
	If there are no more events in the queue then #Poll returns @Null.
	End Rem
	Function Poll:TColEvent()
		If queue.IsEmpty()
			current=Null
		Else
			current=TColEvent(queue.First())
			queue.RemoveFirst
		EndIf
		Return current
	End Function
	
	Rem
	bbdoc: Gets the next collision event from the queue.
	returns: The next collision event.
	about: Similar to #Poll, however #Peek does not remove the event from the queue.
	End Rem
	Function Peek:TColEvent()
		If queue.IsEmpty() Then Return Null
		Return TColEvent(queue.First())
	End Function
	
	Rem
	bbdoc: Gets the collision event associated with two objects.
	returns: The collision event found.
	about: #Seek finds the collision event associated with two objects.
	One of the parameters must be a TMass the other can be either a TMass or TZone.
	Alternatively leaving the second parameter as @Null will check for a collision with the simulation bounds.
	The returned collision event will not be removed from the queue,
	and collisions that have already been polled can still be found using #Seek.
	If no collision occured between the two objects then #Seek returns @Null.
	End Rem
	Function Seek:TColEvent( colObj1:Object, colObj2:Object=Null )
		Local obj1:Object
		Local obj2:Object
		If colObj2=Null Then colObj2=phys
		If colObj1.Compare(colObj2)<0
			obj1=colObj1
			obj2=colObj2
		Else
			obj1=colObj2
			obj2=colObj1
		EndIf
		Local map2:TMap=TMap(map.ValueForKey(obj1))
		If map2
			Return TColEvent(map2.ValueForKey(obj2))
		Else
			Return Null
		EndIf
	End Function
	
	Rem
	bbdoc: Clears the collision event queue.
	about: After a call to #Flush the functions #Poll, #Peek, #Seek and #GetCurrent will all return @Null.
	#TPhysics.Update will automatically #Flush the collision event queue ready to make a new one.
	End Rem
	Function Flush()
		queue.Clear
		map.Clear
		current=Null
	End Function
	
	Rem
	bbdoc: Gets the last polled collision event.
	returns: the last polled collision event.
	about: If no collision event has been polled yet or an empty queue has been polled then #GetCurrent returns @Null.
	End Rem
	Function GetCurrent:TColEvent()
		Return current
	End Function
	
	Rem
	bbdoc: Gets the ID of the collision object.
	returns: The ID of the collision object.
	about: The return value can be checked against the constants, @MASS_MASS, @MASS_ZONE, @MASS_BOUNDS and @MASS_CUSTOM.
	A value of @MASS_MASS means the collision event is between two TMass objects.
	A value of @MASS_MASS means the event is between a TMass and a TZone.
	@MASS_BOUNDS means the events is between a TMass and the simulation bounds.
	If the collision event is does not fit any of these values then #GetID return @MASS_CUSTOM.
	End Rem
	Method GetID:Int()
		Return id
	End Method
	
	Rem
	bbdoc: Gets the first TMass associated with the collision event.
	returns: The first TMass associated with the collision event.
	about: Collision events must always have at least one TMass associated with them.
	Use #GetMass1 to retrieve that TMass.
	End Rem
	Method GetMass1:TMass()
		Return obj1
	End Method
	
	Rem
	bbdoc: Gets the second TMass associated with the collision event.
	returns: The second TMass associated with the collision event.
	about: If there isn't a second TMass associated with the collision event then #GetMass2 returns @Null.
	End Rem
	Method GetMass2:TMass()
		Return TMass(obj2)
	End Method
	
	Rem
	bbdoc: Gets the TZone associated with the collision event.
	returns: The TZone associated with the collision event.
	about: If no TZone is associated with the collision event then #GetZone returns @Null.
	End Rem
	Method GetZone:TZone()
		Return TZone(obj2)
	End Method
	
	Rem
	bbdoc: Get the x coordinate of the collision event.
	returns: The x coordinate of the collision.
	End Rem
	Method GetX:Float()
		Return x
	End Method
	
	Rem
	bbdoc: Get the y coordinate of the collision event.
	returns: The y coordinate of the collision.
	End Rem
	Method GetY:Float()
		Return y
	End Method
	
	Rem
	bbdoc: Get the force of the collision event.
	returns: The force of the collision.
	about: Because PhysLite uses a verlet integration collisions aren't resolved immedaitely.
	They are in fact relaxed over several iterations, and even sometimes several updates.
	As a result the force is shared through these relaxations.
	Using #SetMode to set the mode to @MODE_UPDATE will total all the forces over several iterations,
	but it should be noted that it will not include force shared across several updates will not be included.
	The force returned is not just the result of kinetic(velocity) force, but also the result of weight.
	A stationary TMass, which is under many other TMass objects,
	could easily create a collision with more force than one created solely by movement.
	End Rem
	Method GetForce:Float()
		Return force
	End Method
End Type

Private

Const PHYS_MAX_FLOAT:Float=+10^38
Const PHYS_MIN_FLOAT:Float=0.0001

Global phys:TPhysics

Function LinesCross:Int(x1#,y1#,x2#,y2#,x3#,y3#,x4#,y4#)
	'New LinesCross code thanks to Jake
	Local dpx#= x3 - x1 + x4 - x2
	Local dpy#= y3 - y1 + y4 - y2
	Local qax#= x2 - x1
	Local qay#= y2 - y1
	Local qbx#= x4 - x3
	Local qby#= y4 - y3
	
	Local d#= qay*qbx - qby*qax
	Local la# = qbx*dpy - qby*dpx
	Local lb# = qax*dpy - qay*dpx
	Return (Abs(la)<=Abs(d) And Abs(lb)<=Abs(d))
Rem
( x0:Float, y0:Float , x1:Float, y1:Float,..
						x2:Float ,y2:Float, x3:Float, y3:Float )
	  
	Local n:Float=(y0-y2)*(x3-x2)-(x0-x2)*(y3-y2)
	Local d:Float=(x1-x0)*(y3-y2)-(y1-y0)*(x3-x2)
	
	If Abs(d) < 0.0001 
		' Lines are parallel!
		Return False
	Else
		' Lines might cross!
		Local Sn:Float=(y0-y2)*(x1-x0)-(x0-x2)*(y1-y0)

		Local AB:Float=n/d
		If AB>0.0 And AB<1.0
			Local CD:Float=Sn/d
			If CD>0.0 And CD<1.0
				Return True
			End If
		End If
	
		' Lines didn't cross, because the intersection was beyond the end points of the lines
	EndIf

	' Lines do Not cross!
	Return False
EndRem
End Function

Type TQuadTree
	Field posX:Float, posY:Float
	Field sizeX:Float, sizeY:Float
	
	Field childList:TList=New TList
	
	Function Create:TQuadTree( area:Float[], style:Int, x:Float, y:Float, width:Float, height:Float, level:Int )
		Local col:Int=False
		
		Select style
		Case TZone.LINE	'LINE
			Local x1:Float=area[0]
			Local y1:Float=area[1]
			If x1>x And x1<x+width And y1>y And y1<y+height
				col=True
			EndIf
			For Local count:Int=2 Until area.Length Step 2
				Local x2:Float=area[count]
				Local y2:Float=area[count+1]
				If x2>x And x2<x+width And y2>y And y2<y+height
					col=True
				EndIf
				If LinesCross(x1,y1,x2,y2,x,y,x+width,y)
					col=True
				EndIf
				If LinesCross(x1,y1,x2,y2,x,y,x,y+height)
					col=True
				EndIf
				If LinesCross(x1,y1,x2,y2,x+width,y,x+width,y+height)
					col=True
				EndIf
				If LinesCross(x1,y1,x2,y2,x,y+height,x+width,y+height)
					col=True
				EndIf
				
				x1=x2
				y1=y2
			Next
		Case TZone.LOOP	'LOOP
			Local x1:Float=area[area.Length-2]
			Local y1:Float=area[area.Length-1]
			For Local count:Int=0 Until area.Length Step 2
				Local x2:Float=area[count]
				Local y2:Float=area[count+1]
				If x2>x And x2<x+width And y2>y And y2<y+height
					col=True
				EndIf
				If LinesCross(x1,y1,x2,y2,x,y,x+width,y)
					col=True
				EndIf
				If LinesCross(x1,y1,x2,y2,x,y,x,y+height)
					col=True
				EndIf
				If LinesCross(x1,y1,x2,y2,x+width,y,x+width,y+height)
					col=True
				EndIf
				If LinesCross(x1,y1,x2,y2,x,y+height,x+width,y+height)
					col=True
				EndIf
				
				x1=x2
				y1=y2
			Next
		Case TZone.FILL	'FILL
			Local x1:Float=area[area.Length-2]
			Local y1:Float=area[area.Length-1]
			Local cur_quad:Int
			If x1<x
				If y1<y
					cur_quad=1
				Else
					cur_quad=4
				EndIf
			Else
				If y1<y
					cur_quad=2
				Else
					cur_quad=3
				EndIf	
			EndIf
			Local total:Int
			
			For Local count:Int=0 Until area.Length Step 2
				Local x2:Float=area[count]
				Local y2:Float=area[count+1]
				Local next_quad:Int
				If x2<x
					If y2<y
						next_quad=1
					Else
						next_quad=4
					EndIf
				Else
					If y2<y
						next_quad=2
					Else
						next_quad=3
					EndIf	
				EndIf
				Local diff:Int=next_quad-cur_quad
				
				Select diff
				Case 2,-2
					If ( x2 - ( ((y2 - y) * (x1 - x2)) / (y1 - y2) ) )<x
						diff=-diff
					EndIf
				Case 3
					diff=-1
				Case -3
					diff=1
				End Select
				
				total:+diff
				cur_quad=next_quad
				
				If x2>x And x2<x+width And y2>y And y2<y+height
					col=True
				EndIf
				If LinesCross(x1,y1,x2,y2,x,y,x+width,y)
					col=True
				EndIf
				If LinesCross(x1,y1,x2,y2,x,y,x,y+height)
					col=True
				EndIf
				If LinesCross(x1,y1,x2,y2,x+width,y,x+width,y+height)
					col=True
				EndIf
				If LinesCross(x1,y1,x2,y2,x,y+height,x+width,y+height)
					col=True
				EndIf
				
				x1=x2
				y1=y2
			Next
			
			If Abs(total)=4
				col=True
			EndIf
		Case TZone.HOLE	'HOLE
			Local x1:Float=area[area.Length-2]
			Local y1:Float=area[area.Length-1]
			Local cur_quad:Int
			If x1<x
				If y1<y
					cur_quad=1
				Else
					cur_quad=4
				EndIf
			Else
				If y1<y
					cur_quad=2
				Else
					cur_quad=3
				EndIf	
			EndIf
			Local total:Int
			
			For Local count:Int=0 Until area.Length Step 2
				Local x2:Float=area[count]
				Local y2:Float=area[count+1]
				Local next_quad:Int
				If x2<x
					If y2<y
						next_quad=1
					Else
						next_quad=4
					EndIf
				Else
					If y2<y
						next_quad=2
					Else
						next_quad=3
					EndIf	
				EndIf
				Local diff:Int=next_quad-cur_quad
				
				Select diff
				Case 2,-2
					If ( x2 - ( ((y2 - y) * (x1 - x2)) / (y1 - y2) ) )<x
						diff=-diff
					EndIf
				Case 3
					diff=-1
				Case -3
					diff=1
				End Select
				
				total:+diff
				cur_quad=next_quad
				
				If x2>x And x2<x+width And y2>y And y2<y+height
					col=True
				EndIf
				If LinesCross(x1,y1,x2,y2,x,y,x+width,y)
					col=True
				EndIf
				If LinesCross(x1,y1,x2,y2,x,y,x,y+height)
					col=True
				EndIf
				If LinesCross(x1,y1,x2,y2,x+width,y,x+width,y+height)
					col=True
				EndIf
				If LinesCross(x1,y1,x2,y2,x,y+height,x+width,y+height)
					col=True
				EndIf
				
				x1=x2
				y1=y2
			Next
			
			If Abs(total)<>4
				col=True
			EndIf
		End Select
		
		If col
			Local qt:TQuadTree=New TQuadTree
			
			qt.posX=x
			qt.posY=y
			qt.sizeX=width
			qt.sizeY=height
			
			level:-1
			width:/2
			height:/2
			If level>0	'Make children
				Local childCount:Int=0
				Local child:TQuadTree=TQuadTree.Create(area,style,x,y,width,height,level)
				If child
					qt.childList.AddLast child
					If child.childList.IsEmpty() Then childCount:+1
				EndIf
				child:TQuadTree=TQuadTree.Create(area,style,x+width,y,width,height,level)
				If child
					qt.childList.AddLast child
					If child.childList.IsEmpty() Then childCount:+1
				EndIf
				child:TQuadTree=TQuadTree.Create(area,style,x,y+height,width,height,level)
				If child
					qt.childList.AddLast child
					If child.childList.IsEmpty() Then childCount:+1
				EndIf
				child:TQuadTree=TQuadTree.Create(area,style,x+width,y+height,width,height,level)
				If child
					qt.childList.AddLast child
					If child.childList.IsEmpty() Then childCount:+1
				EndIf
				If childCount=4 Then qt.childList.Clear
			EndIf
			
			Return qt
		Else
			Return Null
		EndIf
	End Function
	
	Method Collide:Int( offsetX:Float, offsetY:Float, x:Float, y:Float, width:Float, hole:Int )
		If (offsetX+posX>x+width) Or (offsetX+posX+sizeX<x) Or (offsetY+posY>y+width) Or (offsetY+posY+sizeY<y)
			Return hole
		Else
			If childList.IsEmpty()
				Return True
			Else
				For Local child:TQuadTree=EachIn childList
					If child.Collide(offsetX,offsetY,x,y,width,False) Then Return True
				Next
			EndIf
		EndIf
		Return False
	End Method
	
	Method Draw( x:Float, y:Float, filled:Int )
		If childList.IsEmpty()
			If filled
				PhysicsDrawRect x+posX,y+posY,sizeX,sizeY
			Else
				PhysicsDrawLine x+posX,y+posY,x+posX+sizeX,y+posY
				PhysicsDrawLine x+posX+sizeX,y+posY,x+posX+sizeX,y+posY+sizeY
				PhysicsDrawLine x+posX+sizeX,y+posY+sizeY,x+posX,y+posY+sizeY
				PhysicsDrawLine x+posX,y+posY+sizeY,x+posX,y+posY
				PhysicsDrawLine x+posX,y+posY,x+posX+sizeX,y+posY+sizeY
				PhysicsDrawLine x+posX,y+posY+sizeY,x+posX+sizeX,y+posY
			EndIf
		Else
			For Local qt:TQuadTree=EachIn childList
				qt.Draw x,y,filled
			Next
		EndIf
	End Method
	
	Method Write( stream:TStream )
		WriteFloat stream,posX
		WriteFloat stream,posY
		WriteFloat stream,sizeX
		WriteFloat stream,sizeY
		WriteInt stream,childList.Count()
		For Local child:TQuadTree=EachIn childList
			child.Write stream
		Next
		
	End Method
	
	Function Read:TQuadTree( stream:TStream )
		Local qt:TQuadTree=New TQuadTree
		qt.posX=ReadFloat(stream)
		qt.posY=ReadFloat(stream)
		qt.sizeX=ReadFloat(stream)
		qt.sizeY=ReadFloat(stream)
		Local children:Int=ReadInt(stream)
		For Local count:Int=1 To children
			qt.childList.AddLast TQuadTree.Read(stream)
		Next
		
		Return qt
	End Function
End Type

