'Procedural functions

Rem
	Each function is directly named for it's corresponding function with the
	first parameter being the object to be affected. For example,
	mass.Move(x,y) becomes MassMove(mass,x,y). For now use the OO docs, I'll
	see about documenting all the functions sometime in the future.
EndRem

'Physics functions

Rem
bbdoc: Creates a new physics simulation.
returns: A TPhysics object.
about: Creates a new physics simulation, which can be stepped through using #PhysicsUpdate.

%Note: that the gravity @gravY parameter is before the @gravX parameter.

See Also: #PhysicsUpdate, #PhysicsSetBounds, #PhysicsSetGravity, #PhysicsSetIterations and #PhysicsEnd
End Rem
Function PhysicsBegin:TPhysics( width:Float, height:Float, gravY:Float, gravX:Float=0.0, i:Int=1 )
	Return TPhysics.Create(width,height,gravY,gravX,i)
End Function

Rem
bbdoc: Update simulation
about:
Process the physics iterations and responses.
#PhysicsUpdate will #Flush the event collision queue, and repopulate it.

See Also: #PhysicsBegin and #PhysicsEnd
End Rem
Function PhysicsUpdate()
	TPhysics.Update
End Function

Rem
bbdoc: Ends the physics simulation.
about:
See Also: #PhysicsBegin
End Rem
Function PhysicsEnd()
	TPhysics.Free
End Function

Rem
bbdoc: Sets the simulation gravity.
about:
Gravity can go in any direction dictated by the vector @x, @y.
End Rem
Function PhysicsSetGravity( x:Float, y:Float )
	TPhysics.SetGravity x,y
End Function

Rem
bbdoc: The number of iterations used by the simulation.
about:
The higher the @value the more accurate and stiff the simulation.
The lower the @value then the faster and softer the simulation.
End Rem
Function PhysicsSetIterations( i:Int )
	TPhysics.SetIterations i
End Function

Rem
bbdoc: Set the upper and lower bounds of the simulation.
about:
See Also: #PhysicsGetBounds and #PhysicsSetFriction
End Rem
Function PhysicsSetBounds( lowerX:Float, lowerY:Float, upperX:Float, upperY:Float )
	TPhysics.SetBounds lowerX,lowerY,upperX,upperY
End Function

Rem
bbdoc: Get the upper and lower bounds of the simulation.
about:
See Also: #PhysicsSetBounds
End Rem
Function PhysicsGetBounds( lowerX:Float Var, lowerY:Float Var, upperX:Float Var, upperY:Float Var )
	TPhysics.GetBounds lowerX,lowerY,upperX,upperY
End Function

Rem
bbdoc: Set the friction of the simulations bounds.
about:
See Also: #PhysicsSetBounds
End Rem
Function PhysicsSetFriction( frict:Float )
	TPhysics.SetFriction frict
End Function

Rem
bbdoc: Activate or de-activate the use of the shifted grid for #TMass occlusion.
about:
Set to %true to use the shifted grid to occlude #TMass objects.
The shifted grid occlusion is not used by default.
The shifted grid gives the best performance when used with extremely large simulations,
and will actually be slower on very small simulations. For more information see #PhysicsSetGridSize.

See Also: #PhysicsSetGridSize
EndRem
Function PhysicsUseSGrid( value:Int )
	TPhysics.UseShiftedGrid value
End Function

Rem
bbdoc: Set the size of the shifted grid.
about:
The shifted grid size can be in the range of 8-64.
When using the shifted grid, setting the size of the grid too high can make the simulation unstable.
To make sure that the grid is not set too high use the following equation, (maxBound-minBound)/diam,
Where minBound and maxBound are the dimensions of the smallest side of the simulation area,
and diam is the diameter(radius*2) of the largest TMass in the simulation.
Even this equation will create a shifted grid size right on the limit
so it is probably best to use this equation as an absolute maximum.

See Also: #PhysicsUseSGrid
EndRem
Function PhysicsSetGridSize( value:Int )
	TPhysics.SetGridSize value
End Function

Rem
bbdoc: Set the area of simulation to draw.
about: 
@centerX and @centerY are the simulation coordinates to be mapped to the center of the screen.

See Also: #PhysicsSetDrawZoom, #PhysicsSetDrawRotation, #PhysicsUpdateTransform, #PhysicsMouseX,
#PhysicsMouseY, #PhysicsSimX, #PhysicsSimY, #PhysicsDrawImage, #PhysicsDrawPoly, #PhysicsDrawRect,
#PhysicsDrawLine, #PhysicsDrawOval, #PhysicsScreenX and #PhysicsScreenY
End Rem
Function PhysicsSetDrawCoords( centerX:Float, centerY:Float )
	TPhysics.SetDrawCoords centerX,centerY
End Function

Rem
bbdoc: Set the zoom of the draw commands.
about:
PhysicsSetDrawZoom scales all the physics drawing functions by @zoom.

See Also: #PhysicsSetDrawCoords, #PhysicsSetDrawRotation, #PhysicsUpdateTransform, #PhysicsMouseX,
#PhysicsMouseY, #PhysicsSimX, #PhysicsSimY, #PhysicsDrawImage, #PhysicsDrawPoly, #PhysicsDrawRect,
#PhysicsDrawLine, #PhysicsDrawOval, #PhysicsScreenX and #PhysicsScreenY
End Rem
Function PhysicsSetDrawZoom( zoom:Float=1.0 )
	TPhysics.SetDrawZoom zoom
End Function

Rem
bbdoc: Sets the angle at which to draw the simulation.
about:
SetDrawRotation rotates all drawing commands in PhysLite.
Use the drawing commands within PhysLite to draw in the corrrect simulation location.
SetDrawRotation does not affect the ordinary Max2D drawing commands.

See Also: #PhysicsSetDrawZoom, #PhysicsSetDrawCoords, #PhysicsUpdateTransform, #PhysicsMouseX,
#PhysicsMouseY, #PhysicsSimX, #PhysicsSimY, #PhysicsDrawImage, #PhysicsDrawPoly, #PhysicsDrawRect,
#PhysicsDrawLine, #PhysicsDrawOval, #PhysicsScreenX and #PhysicsScreenY
End Rem
Function PhysicsSetDrawRotation( angle:Float=0.0 )
	TPhysics.SetDrawRotation angle
End Function

Rem
bbdoc: Updates the transform/rotation matrix.
about:
Under normal circamstances PhysicsUpdateTransform will not be needed 
as the transform/rotation matrix is update by any commands that would change it.
However, PhysicsUpdateTransform is required when the display dimensions are changed,
and it becomes indespensable when using a MaxGUI canvas.

See Also: #PhysicsSetDrawCoords, #PhysicsSetDrawZoom, #PhysicsSetDrawRotation, #PhysicsMouseX,
#PhysicsMouseY, #PhysicsSimX, #PhysicsSimY, #PhysicsDrawImage, #PhysicsDrawPoly, #PhysicsDrawRect,
#PhysicsDrawLine, #PhysicsDrawOval, #PhysicsScreenX and #PhysicsScreenY
End Rem
Function PhysicsUpdateTransform()
	TPhysics.UpdateTransform
End Function

Rem
bbdoc: Gets the mouse X coordinate.
returns: Mouse X coordinate.
about:
MouseX returns the simulation X coordinate of the mouse cursor.

See Also: #PhysicsSetDrawCoords, #PhysicsSetDrawZoom, #PhysicsSetDrawRotation, #PhysicsUpdateTransform,
#PhysicsMouseY, #PhysicsSimX, #PhysicsSimY, #PhysicsScreenX and #PhysicsScreenY
End Rem
Function PhysicsMouseX:Float()
	Return TPhysics.MouseX()
End Function

Rem
bbdoc: Gets the mouse Y coordinate.
returns: Mouse Y coordinate.
about:
MouseY returns the simulation Y coordinate of the mouse cursor.

See Also: #PhysicsSetDrawCoords, #PhysicsSetDrawZoom, #PhysicsSetDrawRotation, #PhysicsUpdateTransform,
#PhysicsMouseX, #PhysicsSimX, #PhysicsSimY, #PhysicsScreenX and #PhysicsScreenY
End Rem
Function PhysicsMouseY:Float()
	Return TPhysics.MouseY()
End Function

Rem
bbdoc: Gets the simulation X coordinate of a screen x,y location.
returns: Simulation X coordinate.
about:
See Also: #PhysicsSetDrawCoords, #PhysicsSetDrawZoom, #PhysicsSetDrawRotation, #PhysicsUpdateTransform,
#PhysicsMouseX, #PhysicsMouseY, #PhysicsSimY, #PhysicsDrawOval, #PhysicsScreenX and #PhysicsScreenY
End Rem
Function PhysicsSimX:Float( x:Float, y:Float )
	Return TPhysics.SimX(x,y)
End Function

Rem
bbdoc: Gets the simulation Y coordinate of a screen x,y location.
returns: Simulation Y coordinate.
about:
See Also: #PhysicsSetDrawCoords, #PhysicsSetDrawZoom, #PhysicsSetDrawRotation, #PhysicsUpdateTransform,
#PhysicsMouseX, #PhysicsMouseY, #PhysicsSimX, #PhysicsScreenX and #PhysicsScreenY
End Rem
Function PhysicsSimY:Float( x:Float, y:Float )
	Return TPhysics.SimY(x,y)
End Function

Rem
bbdoc: Draws an image.
about:
Draws an image at simulation coordinates x,y and with the correct rotation.

See Also: #PhysicsSetDrawCoords, #PhysicsSetDrawZoom, #PhysicsSetDrawRotation, #PhysicsUpdateTransform,
#PhysicsDrawPoly, #PhysicsDrawRect, #PhysicsDrawLine and #PhysicsDrawOval
End Rem
Function PhysicsDrawImage( image:TImage, x:Float, y:Float, frame:Int=0 )
	TPhysics.DrawImage image,x,y,frame
End Function

Rem
bbdoc: Draws a polygon.
about:
Draws a polygon at simulation coordinates x,y and with the correct rotation.

See Also: #PhysicsSetDrawCoords, #PhysicsSetDrawZoom, #PhysicsSetDrawRotation, #PhysicsUpdateTransform,
#PhysicsDrawImage, #PhysicsDrawRect, #PhysicsDrawLine and #PhysicsDrawOval
End Rem
Function PhysicsDrawPoly( xy:Float[], x:Float=0.0, y:Float=0.0 )
	TPhysics.DrawPoly xy,x,y
End Function

Rem
bbdoc: Draws a line.
about:
Draws a line at the correct simulation coordinates.

See Also: #PhysicsSetDrawCoords, #PhysicsSetDrawZoom, #PhysicsSetDrawRotation, #PhysicsUpdateTransform,
#PhysicsDrawImage, #PhysicsDrawPoly, #PhysicsDrawRect and #PhysicsDrawOval
End Rem
Function PhysicsDrawLine( x:Float, y:Float, x2:Float, y2:Float )
	TPhysics.DrawLine x,y,x2,y2
End Function

Rem
bbdoc: Draws an oval.
about:
Draws an oval at simulation coordinates x,y and with the correct rotation.

See Also: #PhysicsSetDrawCoords, #PhysicsSetDrawZoom, #PhysicsSetDrawRotation, #PhysicsUpdateTransform,
#PhysicsDrawImage, #PhysicsDrawPoly, #PhysicsDrawRect and #PhysicsDrawLine
End Rem
Function PhysicsDrawOval( x:Float, y:Float, width:Float, height:Float )
	TPhysics.DrawOval x,y,width,height
End Function

Rem
bbdoc: Draws a rectangle.
about:
Draws a rectangle at simulation coordinates x,y and with the correct rotation.

See Also: #PhysicsSetDrawCoords, #PhysicsSetDrawZoom, #PhysicsSetDrawRotation, #PhysicsUpdateTransform,
#PhysicsDrawImage, #PhysicsDrawPoly, #PhysicsDrawLine and #PhysicsDrawOval
End Rem
Function PhysicsDrawRect( x:Float, y:Float, width:Float, height:Float )
	TPhysics.DrawRect x,y,width,height
End Function

Rem
bbdoc: Gets the screen x coordinate.
returns: The screen x coordinate of simulation x,y coordinate.
about:
See Also: #PhysicsSetDrawCoords, #PhysicsSetDrawZoom, #PhysicsSetDrawRotation, #PhysicsUpdateTransform,
#PhysicsMouseX, #PhysicsMouseY, #PhysicsSimX, #PhysicsSimY and #PhysicsScreenY
End Rem
Function PhysicsScreenX:Float( valueX:Float, valueY:Float )
	Return TPhysics.ScreenX(valueX,valueY)
End Function

Rem
bbdoc: Gets the screen y coordinate.
returns: The screen y coordinate of simulation x,y coordinate.
about:
See Also: #PhysicsSetDrawCoords, #PhysicsSetDrawZoom, #PhysicsSetDrawRotation, #PhysicsUpdateTransform,
#PhysicsMouseX, #PhysicsMouseY, #PhysicsSimX, #PhysicsSimY and #PhysicsScreenX
End Rem
Function PhysicsScreenY:Float( valueX:Float, valueY:Float )
	Return TPhysics.ScreenY(valueX,valueY)
End Function

'Mass functions

Rem
bbdoc: Creates a mass object.
returns: Created TMass.
about:
The Create function creates a TMass to be used by the simulation.
The #TMass does not become active until it is assigned to an active #TConstraint.
The parameter @colGroup is the masses collision group.
A value of -1 will set the masses collision group to the next unique value.
If two collidable objects have the same collision group then they will @not collide with each other.
If @colGroup is set to 0 then the mass will @not collide with any other object.
End Rem
Function MassCreate:TMass( x:Float, y:Float, mass:Float, radius:Float, friction:Float, colGroup:Int=-1 )
	Return TMass.Create( x, y, mass, radius, friction, colGroup)
End Function
	
Rem
bbdoc: Counts the number of active #TMass objects.
returns: The number of active #TMass objects.
about: MassCountActive only counts #TMass objects that are associated with an active #TConstraint.
End Rem
Function MassCountActive:Int()
	Return TMass.CountActive()
End Function

Rem
bbdoc: Maps one function to all active TMass objects.
about: ApplyFunc is the function that will be run for every active TMass in the simulation.
End Rem
Function MassApplyToActive( ApplyFunc( mass:TMass ) )
	TMass.ApplyToActive ApplyFunc
End Function

Function MassCopy:TMass( mass:TMass )
	Return mass.Copy()
End Function

Function MassSetName( mass:TMass, n:String )
	mass.SetName n
End Function

Function MassGetName:String( mass:TMass )
	Return mass.GetName()
End Function

Function MassMove( mass:TMass, x:Float, y:Float )
	mass.Move x,y
End Function

Function MassTranslate( mass:TMass, x:Float, y:Float )
	mass.Translate x,y
End Function

Function MassPosition( mass:TMass, x:Float, y:Float )
	mass.Position x,y
End Function

Function MassGetPositionX:Float( mass:TMass )
	Return mass.GetPositionX()
End Function

Function MassGetPositionY:Float( mass:TMass )
	Return mass.GetPositionY()
End Function

Function MassGetPosition( mass:TMass, x:Float Var, y:Float Var )
	mass.GetPosition x,y
End Function

Function MassGetX:Float( mass:TMass )
	Return mass.GetX()
End Function

Function MassGetY:Float( mass:TMass )
	Return mass.GetY()
End Function

Function MassSetMass( mass:TMass, m:Float )
	mass.SetMass m
End Function

Function MassGetMass:Float( mass:TMass )
	Return mass.GetMass()
End Function

Function MassSetRadius( mass:TMass, r:Float )
	mass.SetRadius r
End Function

Function MassGetRadius:Float( mass:TMass )
	Return mass.GetRadius()
End Function

Function MassSetFriction( mass:TMass, f:Float )
	mass.SetFriction f
End Function

Function MassGetFriction:Float( mass:TMass )
	Return mass.GetFriction()
End Function

Function MassFreeze( mass:TMass )
	mass.Freeze
End Function

Function MassThaw( mass:TMass )
	mass.Thaw
End Function

Function MassIsFrozen:Int( mass:TMass )
	Return mass.IsFrozen()
End Function

Function MassSetVelocity( mass:TMass, x:Float, y:Float )
	mass.SetVelocity x,y
End Function

Function MassAddVelocity( mass:TMass, x:Float, y:Float )
	mass.AddVelocity x,y
End Function

Function MassGetVelocityX:Float( mass:TMass )
	Return mass.GetVelocityX()
End Function

Function MassGetVelocityY:Float( mass:TMass )
	Return mass.GetVelocityY()
End Function

Function MassGetVX:Float( mass:TMass )
	Return mass.GetVX()
End Function

Function MassGetVY:Float( mass:TMass )
	Return mass.GetVY()
End Function

Function MassGetVelocity:Float( mass:TMass )
	Return mass.GetVelocity()
End Function

Function MassGetColGroup:Int( mass:TMass )
	Return mass.GetColGroup()
End Function

Function MassSetColGroup( mass:TMass, value:Int=-1 )
	mass.SetColGroup value
End Function

Function MassIsActive:Int( mass:TMass )
	Return mass.IsActive()
End Function

Function MassGetIndex:Int( mass:TMass )
	Return mass.GetIndex()
End Function

Function MassGetAdvBody:TAdvBody( mass:TMass )
	Return mass.GetAdvBody()
End Function

Function MassDraw( mass:TMass )
	mass.Draw
End Function

'Constraint functions

Rem
bbdoc: Creates a new TConstraint object.
returns: Created TConstraint.
about:
A constraint is an invisible conection between two TMasses.
@restLength is the length of the constraint.
Setting @restLength to -1 results in @restLength being calculated based on the current distance between the two masses.
End Rem
Function ConCreate:TConstraint( m1:TMass, m2:TMass, restLength:Float=-1, stiffness:Float=1.0 )
	Return TConstraint.Create(m1,m2,restLength,stiffness)
End Function

Rem
bbdoc: Counts the number of active #TConstraints.
returns: The number of active #TConstraints.
about: ConCountActive only counts #TConstraints that are associated with a #TBody.
End Rem
Function ConCountActive:Int()
	Return TConstraint.CountActive()
End Function

Rem
bbdoc: Maps one function to all active TConstraints.
about: ApplyFunc is the function that will be run for every active TConstraint in the simulation.
End Rem
Function ConApplyToActive( ApplyFunc( con:TConstraint ) )
	TConstraint.ApplyToActive ApplyFunc
End Function

Function ConSetName( con:TConstraint, n:String )
	con.SetName n
End Function

Function ConGetName:String( con:TConstraint )
	Return con.GetName()
End Function

Function ConAssign( con:TConstraint, b:TBody )
	con.Assign b
End Function

Function ConFree( con:TConstraint, b:TBody )
	con.Free b
End Function

Function ConGetMass1:TMass( con:TConstraint )
	Return con.GetMass1()
End Function

Function ConGetMass2:TMass( con:TConstraint )
	Return con.GetMass2()
End Function

Function ConSetRestLength( con:TConstraint, length:Float )
	con.SetRestLength length
End Function

Function ConGetRestLength:Float( con:TConstraint )
	Return con.GetRestLength()
End Function

Function ConSetStiffness( con:TConstraint, stiff:Float )
	con.SetStiffness stiff
End Function

Function ConGetStiffness:Float( con:TConstraint )
	Return con.GetStiffness()
End Function

Function ConGetAngle:Float( con:TConstraint )
	Return con.GetAngle()
End Function

Function ConGetLength:Float( con:TConstraint )
	Return con.GetLength()
End Function

Function ConGetStress:Float( con:TConstraint )
	Return con.GetLength()
End Function

Function ConIsActive:Int( Con:TConstraint )
	Return con.IsActive()
End Function

Function ConGetIndex:Int( con:TConstraint )
	Return con.GetIndex()
End Function

Function ConGetAdvBody:TAdvBody( con:TConstraint )
	Return con.GetAdvBody()
End Function

Function ConDraw( con:TConstraint )
	con.Draw
End Function

'Body functions
Function BodyCreate:TBody()
	Return New TBody
End Function

Function BodySetName( body:TBody, n:String )
	body.SetName n
End Function

Function BodyGetName:String( body:TBody )
	Return body.GetName()
End Function

Function BodyGetConstraints:TList( body:TBody )
	Return body.GetConstraints()
End Function

Function BodyFree( body:TBody )
	body.Free
End Function

Function BodyClear( body:TBody )
	body.Clear
End Function

'Advanced Body functions
Rem
bbdoc: Creates an empty TAdvBody.
returns: The created TAdvBody.
End Rem
Function AdvBodyCreate:TAdvBody()
	Return New TAdvBody
End Function

Rem
bbdoc: Loads a previously saved advanced body object.
returns: Loaded TAdvBody.
about: 
The Load function loads a #TAdvBody that has previously been saved using the Save method.
The @active parameter determines if the #TAdvBody will be Activated immediately after loading.

The loaded #TAdvBody will be identical to the original,
however collision groups need a little further explination.
If a mass was created using an absolute value for it's @colGroup, i.e. 0 or above,
then the corrisponding loaded mass will have the exact same colGroup.
If however the original mass was created using -1 as it's colGroup the loaded mass will be given the next unique colGroup.
Masses within the same #TAdvBody that share the same colGroup will keep those associations after loading.
End Rem
Function AdvBodyLoad:TAdvBody( url:Object, active:Int=False )
	Return TAdvBody.Load(url,active)
End Function

Rem
bbdoc: Counts all TAdvBodies.
returns: The number of TAdvBodies.
about: AdvBodyCountAll will include TAdvBodies that are not active.
End Rem
Function AdvBodyCountAll:Int()
	Return TAdvBody.CountAll()
End Function

Rem
bbdoc: Maps one function to all TAdvBodies.
about: ApplyFunc is the function that will be run for every active TAdvBody in the simulation.
End Rem
Function AdvBodyApplyToAll( ApplyFunc( advBody:TAdvBody ) )
	TAdvBody.ApplyToAll ApplyFunc
End Function

Function AdvBodySave( AdvBody:TAdvBody, url:Object )
	AdvBody.Save url
End Function

Function AdvBodyCopy:TAdvBody( AdvBody:TAdvBody )
	Return AdvBody.Copy()
End Function

Function AdvBodyCleanIndex( AdvBody:TAdvBody )
	AdvBody.CleanIndex
End Function

Function AdvBodyActivate( AdvBody:TAdvBody )
	AdvBody.Activate
End Function

Function AdvBodyDeActivate( AdvBody:TAdvBody )
	AdvBody.DeActivate
End Function

Function AdvBodyFree( advBody:TAdvBody )
	advBody.Free
End Function

Function AdvBodyDraw( AdvBody:TAdvBody )
	AdvBody.Draw
End Function

Function AdvBodyMove( AdvBody:TAdvBody, x:Float, y:Float )
	AdvBody.Move x,y
End Function

Function AdvBodyTranslate( AdvBody:TAdvBody, x:Float, y:Float )
	AdvBody.Translate x,y
End Function

Function AdvBodyPosition( AdvBody:TAdvBody, x:Float, y:Float)
	AdvBody.Position x,y
End Function

Function AdvBodyGetPosition( AdvBody:TAdvBody, x:Float Var, y:Float Var )
	AdvBody.GetPosition x,y
End Function

Function AdvBodyGetX:Float( AdvBody:TAdvBody )
	Return advBody.GetX()
End Function

Function AdvBodyGetY:Float( AdvBody:TAdvBody )
	Return advBody.GetY()
End Function

Function AdvBodyRotate( AdvBody:TAdvBody, angle:Float )
	AdvBody.Rotate angle
End Function

Function AdvBodyGetRotation:Float( AdvBody:TAdvBody )
	Return AdvBody.GetRotation()
End Function

Function AdvBodySpin( AdvBody:TAdvBody, angle:Float )
	AdvBody.Spin angle
End Function

Function AdvBodyTurn( AdvBody:TAdvBody, angle:Float )
	AdvBody.Turn angle
End Function

Function AdvBodyFreeze( advBody:TAdvBody )
	advBody.Freeze
End Function

Function AdvBodyThaw( advBody:TAdvBody )
	advBody.Thaw
End Function

Function AdvBodyCollide:TList( AdvBody1:TAdvBody, AdvBody2:TAdvBody )
	Return AdvBody1.CollideAdvBody(AdvBody2)
End Function

Function AdvBodyCollideZone:TList( AdvBody:TAdvBody, zone:TZone )
	Return AdvBody.CollideZone(zone)
End Function

Function AdvBodyCollideBounds:TList( AdvBody:TAdvBody )
	Return AdvBody.CollideBounds()
End Function

Function AdvBodyMassAdd:Int( AdvBody:TAdvBody, x:Float, y:Float, mass:Float, radius:Float, friction:Float, colGroup:Int=-1 )
	Return AdvBody.MassAdd(x,y,mass,radius,friction,colGroup)
End Function
	
Function AdvBodyMassRemove( AdvBody:TAdvBody, index:Int )
	AdvBody.MassRemove index
End Function

Function AdvBodyMassMaxIndex:Int( AdvBody:TAdvBody )
	Return AdvBody.MassMaxIndex()
End Function

Function AdvBodyMassSwapIndex( AdvBody:TAdvBody, index1:Int, index2:Int )
	AdvBody.MassSwapIndex index1,index2
End Function

Function AdvBodyMassCleanIndex( AdvBody:TAdvBody )
	AdvBody.MassCleanIndex
End Function

Function AdvBodyMassExists:Int( AdvBody:TAdvBody, index:Int )
	Return AdvBody.MassExists(index)
End Function

Function AdvBodyMassGet:TMass( AdvBody:TAdvBody, index:Int )
	Return AdvBody.MassGet(index)
End Function

Function AdvBodyMassSetName( AdvBody:TAdvBody, index:Int, name:String )
	AdvBody.MassSetName index,name
End Function

Function AdvBodyMassGetName:String( AdvBody:TAdvBody, index:Int )
	Return AdvBody.MassGetName(index)
End Function

Function AdvBodyMassMove( AdvBody:TAdvBody, index:Int, x:Float, y:Float )
	AdvBody.MassMove index,x,y
End Function

Function AdvBodyMassTranslate( AdvBody:TAdvBody, index:Int, x:Float, y:Float )
	AdvBody.MassTranslate index,x,y
End Function

Function AdvBodyMassPosition( AdvBody:TAdvBody, index:Int, x:Float, y:Float )
	AdvBody.MassPosition index,x,y
End Function

Function AdvBodyMassGetPositionX:Float( AdvBody:TAdvBody, index:Int )
	Return AdvBody.MassGetPositionX(index)
End Function

Function AdvBodyMassGetPositionY:Float( AdvBody:TAdvBody, index:Int )
	Return AdvBody.MassGetPositionY(index)
End Function

Function AdvBodyMassGetPosition( AdvBody:TAdvBody, index:Int, x:Float Var, y:Float Var )
	AdvBody.MassGetPosition index,x,y
End Function

Function AdvBodyMassGetX:Float( AdvBody:TAdvBody, index:Int )
	Return AdvBody.MassGetX(index)
End Function

Function AdvBodyMassGetY:Float( AdvBody:TAdvBody, index:Int )
	Return AdvBody.MassGetY(index)
End Function

Function AdvBodyMassSetMass( AdvBody:TAdvBody, index:Int, m:Float )
	AdvBody.MassSetMass index,m
End Function

Function AdvBodyMassGetMass:Float( AdvBody:TAdvBody, index:Int )
	Return AdvBody.MassGetMass(index)
End Function

Function AdvBodyMassSetRadius( AdvBody:TAdvBody, index:Int, r:Float )
	AdvBody.MassSetRadius index,r
End Function

Function AdvBodyMassGetRadius:Float( AdvBody:TAdvBody, index:Int )
	Return AdvBody.MassGetRadius(index)
End Function

Function AdvBodyMassSetFriction( AdvBody:TAdvBody, Index:Int, f:Float )
	AdvBody.MassSetFriction index,f
End Function

Function AdvBodyMassGetFriction:Float( AdvBody:TAdvBody, Index:Int )
	Return AdvBody.MassGetFriction(index)
End Function

Function AdvBodyMassFreeze( AdvBody:TAdvBody, index:Int )
	AdvBody.MassFreeze index
End Function

Function AdvBodyMassThaw( AdvBody:TAdvBody, index:Int )
	AdvBody.MassThaw index
End Function

Function AdvBodyMassIsFrozen:Int( AdvBody:TAdvBody, index:Int )
	Return AdvBody.MassIsFrozen(index)
End Function

Function AdvBodyMassSetVelocity( AdvBody:TAdvBody, index:Int, x:Float, y:Float )
	AdvBody.MassSetVelocity index,x,y
End Function

Function AdvBodyMassAddVelocity( AdvBody:TAdvBody, index:Int, x:Float, y:Float )
	AdvBody.MassAddVelocity index,x,y
End Function

Function AdvBodyMassGetVelocityX:Float( AdvBody:TAdvBody, index:Int )
	Return AdvBody.MassGetVelocityX(index)
End Function

Function AdvBodyMassGetVelocityY:Float( AdvBody:TAdvBody, index:Int )
	Return AdvBody.MassGetVelocityY(index)
End Function

Function AdvBodyMassGetVX:Float( AdvBody:TAdvBody, index:Int )
	Return AdvBody.MassGetVX(index)
End Function

Function AdvBodyMassGetVY:Float( AdvBody:TAdvBody, index:Int )
	Return AdvBody.MassGetVY(index)
End Function

Function AdvBodyMassGetVelocity:Float( AdvBody:TAdvBody, index:Int )
	Return AdvBody.MassGetVelocity(index)
End Function

Function AdvBodyMassGetColGroup:Int( AdvBody:TAdvBody, index:Int )
	Return AdvBody.MassGetColGroup(index)
End Function

Function AdvBodyMassSetColGroup( AdvBody:TAdvBody, index:Int, value:Int=-1 )
	AdvBody.MassSetColGroup index,value
End Function

Function AdvBodyMassIsActive:Int( advBody:TAdvBody, index:Int)
	Return advBody.MassIsActive(index)
End Function

Function AdvBodyMassDraw( AdvBody:TAdvBody, index:Int )
	AdvBody.MassDraw index
End Function

Function AdvBodyMassDrawAll( AdvBody:TAdvBody )
	AdvBody.MassDrawAll
End Function

Function AdvBodyConAddAll( AdvBody:TAdvBody )
	AdvBody.ConAddAll
End Function

Function AdvBodyConAdd:Int( AdvBody:TAdvBody, mIndex1:Int, mIndex2:Int, restLength:Float=-1, stiffness:Float=1.0 )
	Return AdvBody.ConAdd(mIndex1,mIndex2,restLength,stiffness)
End Function

Function AdvBodyConRemove( AdvBody:TAdvBody, index:Int )
	AdvBody.ConRemove index
End Function

Function AdvBodyConMaxIndex:Int( AdvBody:TAdvBody )
	Return AdvBody.ConMaxIndex()
End Function

Function AdvBodyConSwapIndex( AdvBody:TAdvBody, index1:Int, index2:Int )
	AdvBody.ConSwapIndex index1,index2
End Function

Function AdvBodyConCleanIndex( AdvBody:TAdvBody )
	AdvBody.ConCleanIndex
End Function

Function AdvBodyConExists:Int( AdvBody:TAdvBody, index:Int )
	Return AdvBody.ConExists(index)
End Function

Function AdvBodyConGet:TConstraint( AdvBody:TAdvBody, index:Int )
	Return AdvBody.ConGet(index)
End Function

Function AdvBodyConSetName( AdvBody:TAdvBody, index:Int, n:String )
	AdvBody.ConSetName index,n
End Function

Function AdvBodyConGetName:String( AdvBody:TAdvBody, index:Int )
	Return AdvBody.ConGetName(index)
End Function

Function AdvBodyConAssign( AdvBody:TAdvBody, index:Int )
	AdvBody.ConAssign index
End Function

Function AdvBodyConFree( AdvBody:TAdvBody, index:Int )
	AdvBody.ConFree index
End Function

Function AdvBodyConGetMass1:Int( AdvBody:TAdvBody, index:Int )
	Return AdvBody.ConGetMass1(index)
End Function

Function AdvBodyConGetMass2:Int( AdvBody:TAdvBody, index:Int )
	Return AdvBody.ConGetMass2(index)
End Function

Function AdvBodyConSetRestLength( AdvBody:TAdvBody, index:Int, length:Float=-1 )
	AdvBody.ConSetRestLength index,length
End Function

Function AdvBodyConGetRestLength:Float( AdvBody:TAdvBody, index:Int )
	Return AdvBody.ConGetRestLength(index)
End Function

Function AdvBodyConSetStiffness( AdvBody:TAdvBody, index:Int, stiff:Float )
	AdvBody.ConSetStiffness index,stiff
End Function

Function AdvBodyConGetStiffness:Float( AdvBody:TAdvBody, index:Int )
	Return AdvBody.ConGetStiffness(index)
End Function

Function AdvBodyConGetAngle:Float( AdvBody:TAdvBody, index:Int )
	Return AdvBody.ConGetAngle(index)
End Function

Function AdvBodyConGetLength:Float( AdvBody:TAdvBody, index:Int )
	Return AdvBody.ConGetLength(index)
End Function

Function AdvBodyConGetStress:Float( AdvBody:TAdvBody, index:Int )
	Return AdvBody.ConGetStress(index)
End Function

Function AdvBodyConDraw( AdvBody:TAdvBody, index:Int )
	AdvBody.ConDraw index
End Function

Function AdvBodyConDrawAll( AdvBody:TAdvBody )
	AdvBody.ConDrawAll
End Function

Function AdvBodyRigAdd:Int( AdvBody:TAdvBody, iIndex:Int, mIndex:Int, cIndex:Int=-1, angle:Float=0 )
	Return AdvBody.RigAdd(iIndex,mIndex,cIndex,angle)
End Function

Function AdvBodyRigRemove( AdvBody:TAdvBody, index:Int )
	AdvBody.RigRemove index
End Function

Function AdvBodyRigMaxIndex:Int( AdvBody:TAdvBody )
	Return AdvBody.RigMaxIndex()
End Function

Function AdvBodyRigSwapIndex( AdvBody:TAdvBody, index1:Int, index2:Int )
	AdvBody.RigSwapIndex index1,index2
End Function

Function AdvBodyRigCleanIndex( AdvBody:TAdvBody )
	AdvBody.RigCleanIndex
End Function

Function AdvBodyRigExists:Int( AdvBody:TAdvBody, index:Int )
	Return AdvBody.RigExists(index)
End Function

Function AdvBodyRigGet:TRig( AdvBody:TAdvBody, index:Int )
	Return AdvBody.RigGet(index)
End Function

Function AdvBodyRigSetName( AdvBody:TAdvBody, index:Int, n:String )
	AdvBody.RigSetName index,n
End Function

Function AdvBodyRigGetName:String( AdvBody:TAdvBody, index:Int )
	Return AdvBody.RigGetName(index)
End Function

Function AdvBodyRigSetImage( AdvBody:TAdvBody, index:Int, img:Int )
	AdvBody.RigSetImage index,img
End Function

Function AdvBodyRigGetImage:Int( AdvBody:TAdvBody, index:Int )
	Return AdvBody.RigGetImage(index)
End Function

Function AdvBodyRigSetMass( AdvBody:TAdvBody, index:Int, m:Int )
	AdvBody.RigSetMass index,m
End Function

Function AdvBodyRigGetMass:Int( AdvBody:TAdvBody, index:Int )
	Return AdvBody.RigGetMass(index)
End Function

Function AdvBodyRigSetConstraint( AdvBody:TAdvBody, index:Int, c:Int=-1 )
	AdvBody.RigSetConstraint index,c
End Function

Function AdvBodyRigGetConstraint:Int( AdvBody:TAdvBody, index:Int )
	Return AdvBody.RigGetConstraint(index)
End Function

Function AdvBodyRigSetAngle( AdvBody:TAdvBody, index:Int, a:Float )
	AdvBody.RigSetAngle index,a
End Function

Function AdvBodyRigGetAngle:Float( AdvBody:TAdvBody, index:Int )
	Return AdvBody.RigGetAngle(index)
End Function

Function AdvBodyRigSetScale( AdvBody:TAdvBody, index:Int, sclX:Float, sclY:Float )
	AdvBody.RigSetScale index,sclX,sclY
End Function

Function AdvBodyRigGetScale( AdvBody:TAdvBody, index:Int, sclX:Float Var, sclY:Float Var )
	AdvBody.RigGetScale index,sclX,sclY
End Function

Function AdvBodyRigSetColor( AdvBody:TAdvBody, index:Int, red:Int, green:Int, blue:Int )
	AdvBody.RigSetColor index,red,green,blue
End Function

Function AdvBodyRigGetColor( AdvBody:TAdvBody, index:Int, red:Int Var, green:Int Var, blue:Int Var)
	AdvBody.RigGetColor index,red,green,blue
End Function

Function AdvBodyRigSetAlpha( AdvBody:TAdvBody, index:Int, alpha:Float )
	AdvBody.RigSetAlpha index,alpha
End Function

Function AdvBodyRigGetAlpha:Float( AdvBody:TAdvBody, index:Int )
	Return AdvBody.RigGetAlpha(index)
End Function

Function AdvBodyRigSetBlend( AdvBody:TAdvBody, index:Int, blend:Int )
	AdvBody.RigSetBlend index,blend
End Function

Function AdvBodyRigGetBlend:Int( AdvBody:TAdvBody, index:Int )
	Return AdvBody.RigGetBlend(index)
End Function

Function AdvBodyRigDraw( AdvBody:TAdvBody, index:Int )
	AdvBody.RigDraw index
End Function

Function AdvBodyImageAdd:Int( AdvBody:TAdvBody, url:Object )
	Return AdvBody.ImageAdd(url)
End Function

Function AdvBodyImageRemove( AdvBody:TAdvBody, index:Int )
	AdvBody.ImageRemove index
End Function

Function AdvBodyImageMaxIndex:Int( AdvBody:TAdvBody )
	Return AdvBody.ImageMaxIndex()
End Function

Function AdvBodyImageSwapIndex( AdvBody:TAdvBody, index1:Int, index2:Int )
	AdvBody.ImageSwapIndex index1,index2
End Function

Function AdvBodyImageReplace:Int( AdvBody:TAdvBody, index:Int, url:Object )
	Return AdvBody.ImageReplace(index,url)
End Function

Function AdvBodyImageCleanIndex( AdvBody:TAdvBody )
	AdvBody.ImageCleanIndex
End Function

Function AdvBodyImageExists:Int( AdvBody:TAdvBody, index:Int )
	Return AdvBody.ImageExists(index)
End Function

Function AdvBodyImageGet:TImage( AdvBody:TAdvBody, index:Int )
	Return AdvBody.ImageGet(index)
End Function


'Rig functions

Rem
bbdoc: Creates a new TRig
returns: Created TRig
about:
Creates a rig attached to @mass and orientated relative to @con.
@angle is how much the images orientation is offset.
If @con is #Null then the image is orientated only by @angle.
End Rem
Function RigCreate:TRig( image:TImage, mass:TMass, con:TConstraint=Null, angle:Float=0 )
	Return TRig.Create(image,mass,con,angle)
End Function

Function RigSetName( rig:TRig, n:String )
	rig.SetName n
End Function

Function RigGetName:String( rig:TRig )
	Return rig.GetName()
End Function

Function RigSetImage( rig:TRig, img:TImage )
	rig.SetImage img
End Function

Function RigGetImage:TImage( rig:TRig )
	Return rig.GetImage()
End Function

Function RigSetMass( rig:TRig, m:TMass )
	rig.SetMass m
End Function

Function RigGetMass:TMass( rig:TRig )
	Return rig.GetMass()
End Function

Function RigSetConstraint( rig:TRig, c:TConstraint=Null )
	rig.SetConstraint c
End Function

Function RigGetConstraint:TConstraint( rig:TRig )
	Return rig.GetConstraint()
End Function

Function RigSetAngle( rig:TRig, a:Float )
	rig.SetAngle a
End Function

Function RigGetAngle:Float( rig:TRig )
	Return rig.GetAngle()
End Function

Function RigSetScale( rig:TRig, valueX:Float, valueY:Float )
	rig.SetScale valueX,valueY
End Function

Function RigGetScale( rig:TRig, x:Float Var, y:Float Var )
	rig.GetScale x,y
End Function

Function RigSetColor( rig:TRig, r:Int, g:Int, b:Int )
	rig.SetColor r,g,b
End Function

Function RigGetColor( rig:TRig, r:Int Var, g:Int Var, b:Int Var )
	rig.GetColor r,g,b
End Function

Function RigSetAlpha( rig:TRig, value:Float )
	rig.SetAlpha value
End Function

Function RigGetAlpha:Float( rig:TRig )
	Return rig.GetAlpha()
End Function

Function RigSetBlend( rig:TRig, value:Int )
	rig.SetBlend value
End Function

Function RigGetBlend:Int( rig:TRig )
	Return rig.GetBlend()
End Function

Function RigGetIndex:Int( rig:TRig )
	Return rig.GetIndex()
End Function

Function RigGetAdvBody:TAdvBody( rig:TRig )
	Return rig.GetAdvBody()
End Function

Function RigDraw( rig:TRig )
	rig.Draw
End Function

'Zone functions

Rem
bbdoc: Creates a new collision zone.
returns: Collision zone created.
about:
A collision zone is an area of the simulation that masses can collide with.
The collision @area is difined by an array of x:Float,y:Float coordinate pairs.

@Style can be either @LINE, @LOOP, @FILL, or @HOLE.

@colGroup relates to which masses will collide with the zone. See #MassCreate for more information.

@qtLevel is the number of levels in the zones QuadTree.
A value of 1 is the same as having bounding checks and is sufficient in most cases.
QuadTree creation is slow so setting qtLevel too high can cause massive delays in an application.
End Rem
Function ZoneCreate:TZone( area:Float[], style:Int=TZone.LINE, frict:Float=1.0, colGroup:Int=-1, qtLevel:Int=1 )
	Return TZone.Create(area,style,frict,colGroup,qtLevel)
End Function

Rem
bbdoc: Loads a saved collision zone.
returns: Loaded collision zone.
about:
@url is the file where the TZone is saved.
The zones QuadTree is also loaded so does not need to be recalculated.
End Rem
Function ZoneLoad:TZone( url:Object )
	Return TZone.Load(url)
End Function

Rem
bbdoc: Counts the number of active TZones.
returns: The number of active TZones.
about: ZoneCountActive only counts TZones that have been activated.
End Rem
Function ZoneCountActive:Int()
	Return TZone.CountActive()
End Function

Rem
bbdoc: Maps one function to all active TZones.
about: ApplyFunc is the function that will be run for every active TZone in the simulation.
End Rem
Function ZoneApplyToActive( ApplyFunc( zone:TZone ) )
	TZone.ApplyToActive ApplyFunc
End Function

Function ZoneSave( zone:TZone, url:Object, backup:Int=True, packImage:Int=True )
	zone.Save url,backup,packImage
End Function

Function ZoneCopy:TZone( zone:TZone )
	Return zone.Copy()
End Function

Function ZoneActivate( zone:TZone )
	zone.Activate
End Function

Function ZoneDeActivate( zone:TZone )
	zone.DeActivate
End Function

Function ZoneFree( zone:TZone )
	zone.Free
End Function

Function ZoneSetName( zone:TZone, n:String )
	zone.SetName n
End Function

Function ZoneGetName:String( zone:TZone )
	Return zone.GetName()
End Function
	
Function ZoneSetFriction( zone:TZone, f:Float )
	zone.SetFriction f
End Function

Function ZoneGetFriction:Float( zone:TZone )
	Return zone.GetFriction()
End Function

Function ZoneSetStyle( zone:TZone, value:Int )
	zone.SetStyle value
End Function

Function ZoneGetStyle:Int( zone:TZone )
	Return zone.GetStyle()
End Function

Function ZonePosition( zone:TZone, x:Float, y:Float )
	zone.Position x,y
End Function

Function ZoneMove( zone:TZone, x:Float, y:Float )
	zone.Move x,y
End Function

Function ZoneGetPositionX:Float( zone:TZone )
	Return zone.GetPositionX()
End Function

Function ZoneGetPositionY:Float( zone:TZone )
	Return zone.GetPositionY()
End Function

Function ZoneGetX:Float( zone:TZone )
	Return zone.GetX()
End Function

Function ZoneGetY:Float( zone:TZone )
	Return zone.GetY()
End Function

Function ZoneRotate( zone:TZone, value:Float )
	zone.Rotate value
End Function

Function ZoneTurn( zone:TZone, value:Float )
	zone.Turn value
End Function

Function ZoneGetRotation:Float( zone:TZone )
	Return zone.GetRotation()
End Function

Function ZoneSetScale( zone:TZone, x:Float, y:Float )
	zone.SetScale x,y
End Function

Function ZoneGetScale( zone:TZone, x:Float Var, y:Float Var )
	zone.GetScale x,y
End Function

Function ZoneSetColor( zone:TZone, r:Int, g:Int, b:Int )
	zone.SetColor r,g,b
End Function

Function ZoneGetColor( zone:TZone, r:Int Var, g:Int Var, b:Int Var )
	zone.GetColor r,g,b
End Function

Function  ZoneSetAlpha( zone:TZone, value:Float )
	zone.SetAlpha value
End Function

Function ZoneGetAlpha:Float( zone:TZone )
	Return zone.GetAlpha()
End Function

Function ZoneSetBlend( zone:TZone, value:Int )
	zone.SetBlend value
End Function

Function ZoneGetBlend:Int( zone:TZone )
	Return zone.GetBlend()
End Function

Function ZoneVertCount:Int( zone:TZone )
	Return zone.VertCount()
End Function

Function ZoneVertAddLast( zone:TZone, x:Float, y:Float )
	zone.VertAddLast x,y
End Function

Function ZoneVertAddFirst( zone:TZone, x:Float, y:Float )
	zone.VertAddFirst x,y
End Function

Function ZoneVertInsert( zone:TZone, index:Int, x:Float, y:Float )
	zone.VertInsert index,x,y
End Function

Function ZoneVertRemoveLast( zone:TZone )
	zone.VertRemoveLast
End Function

Function ZoneVertRemoveFirst( zone:TZone )
	zone.VertRemoveFirst
End Function

Function ZoneVertRemove( zone:TZone, index:Int )
	zone.VertRemove index
End Function

Function ZoneVertPosition( zone:TZone, index:Int, x:Float, y:Float )
	zone.VertPosition index,x,y
End Function

Function ZoneVertGetPosition( zone:TZone, index:Int, x:Float Var, y:Float Var )
	zone.VertGetPosition index,x,y
End Function

Function ZoneVertGetX:Float( zone:TZone, index:Int )
	Return zone.VertGetX(index)
End Function

Function ZoneVertGetY:Float( zone:TZone, index:Int )
	Return zone.VertGetY(index)
End Function

Function ZoneCalcQuadTree( zone:TZone, level:Int=1 )
	zone.CalcQuadTree level
End Function

Function ZoneGetQTLevel:Int( zone:TZone )
	Return zone.GetQTLevel()
End Function

Function ZoneGetArea:Float[]( zone:TZone )
	Return zone.GetArea()
End Function

Function ZoneSetColGroup( zone:TZone, value:Int=-1 )
	zone.SetColGroup value
End Function

Function ZoneGetColGroup:Int( zone:TZone )
	Return zone.GetColGroup()
End Function

Function ZoneImageSet( zone:TZone, img:TImage )
	zone.ImageSet img
End Function

Function ZoneImageGet:TImage( zone:TZone )
	Return zone.ImageGet()
End Function

Function ZoneImageSetScale( zone:TZone, x:Float, y:Float )
	zone.ImageSetScale x,y
End Function

Function ZoneImageGetScale( zone:TZone, x:Float, y:Float )
	zone.ImageGetScale x,y
End Function

Function ZoneDraw( zone:TZone )
	zone.Draw
End Function

Function ZoneDrawQuadTree( zone:TZone )
	zone.DrawQuadTree
End Function

Function ZoneOverrideCollision( zone:TZone, CustomResponse(zone:TZone,mass:TMass,penNormX:Float,penNormY:Float,penMag:Float)=Null )
	zone.OverrideCollision CustomResponse
End Function

'Collision Event functions

Rem
bbdoc: Creates a collision event object.
returns: The created collision event.
about: 
Craetes a collision event that can then be emitted or posted.

See Also: #ColEventEmit and #ColEventPost
End Rem
Function ColEventCreate:TColEvent( obj1:TMass, obj2:Object, x:Float, y:Float, force:Float )
	Return TColEvent.Create(obj1,obj2,x,y,force)
End Function

Rem
bbdoc: Sets how collision events are added to the event queue
about:
Sets the way collision events are added to the event queue.
If there is more than one iteration then it is possible for two objects to collide more than once during the same update.
#SetMode tells PhysLite how to report multiple collision between the same objects.
@Value can be one of the following values, @MODE_DISCARD, @MODE_UPDATE or @MODE_REPLACE.

[ @MODE_DISCARD | No events are added to the event queue. Hooked functions will still be called. Setting @MODE_DISCARD will give a speed increase. It is also useful for making your own custom  event queue using hook functions.
* @MODE_UPDATE | Totals the force of multiple collisions between the same objects. This is the default mode.
* @MODE_REPLACE | Discard previous collision events between the same two objects and only report the last.
]
End Rem
Function ColEventSetMode( value:Int )
	TColEvent.SetMode value
End Function

Rem
bbdoc: Emits a collision event
about:
When a collision event is emitted all hooked functions are run and then the event is posted to the event queue.
If the hook functions return @Null then the collision event will not be posted to the event queue.
Emitting is usually handled automatically by #PhysicsUpdate but ColEventEmit can be used to fake a collision event.

See Also: #ColEventCreate and #ColEventPost
End Rem
Function ColEventEmit( colEvent:TColEvent )
	TColEvent.Emit colEvent
End Function

Rem
bbdoc: Posts a collision event to the collision events queue
about:
Similar to #ColEventEmit, ColEventPost will add a collision event to the collision event queue.
However posted collision events will not run any hooked functions.
The @update parameter is used to tell PhysLite to either update or replace previous collisions between the same objects.
See #ColEventSetMode for more information on modes.
End Rem
Function ColEventPost( colEvent:TColEvent, update:Int=TColEvent.MODE_UPDATE )
	TColEvent.Post colEvent,update
End Function

Rem
bbdoc: Removes the next collision event from the queue.
returns: The next collision event in the queue.
about:
ColEventPoll sequentially removes collision events from the queue, and sets it to the current event.
If there are no more events in the queue then ColEventPoll returns @Null.

See Also: #ColEventPeek, #ColEventSeek and #CollEventGetCurrent
End Rem
Function ColEventPoll:TColEvent()
	Return TColEvent.Poll()
End Function

Rem
bbdoc: Gets the next collision event from the queue.
returns: The next collision event.
about:
Similar to #ColEventPoll, however ColEventPeek does not remove the event from the queue.

See Also: #ColEventPoll, #ColEventSeek and #CollEventGetCurrent
End Rem
Function ColEventPeek:TColEvent()
	Return TColEvent.Peek()
End Function

Rem
bbdoc: Gets the collision event associated with two objects.
returns: The collision event found.
about: 
ColEventSeek finds the collision event associated with two objects.
One of the parameters must be a #TMass the other can be either a #TMass or #TZone.
Alternatively leaving the second parameter as @Null will check for a collision with the simulation bounds.
The returned collision event will not be removed from the queue,
and collisions that have already been polled can still be found using ColEventSeek.
If no collision occured between the two objects then ColEventSeek returns @Null.

See Also: #ColEventPoll and #ColEventSeek
End Rem
Function ColEventSeek:TColEvent( colObj1:Object, colObj2:Object=Null )
	Return TColEvent.Seek(colObj1,colObj2)
End Function

Rem
bbdoc: Clears the collision event queue.
about: After a call to ColEventFlush the functions #ColEventPoll, #ColEventPeek, #ColEventSeek and #ColEventGetCurrent will all return @Null.
#PhysicsUpdate will automatically ColEventFlush the collision event queue ready to make a new one.
End Rem
Function ColEventFlush()
	TColEvent.Flush
End Function

Rem
bbdoc: Gets the last polled collision event.
returns: the last polled collision event.
about:
If no collision event has been polled yet or an empty queue has been polled then #ColEventGetCurrent returns @Null.
End Rem
Function ColEventGetCurrent:TColEvent()
	Return TColEvent.GetCurrent()
End Function

Function ColEventGetID:Int( colEvent:TColEvent )
	Return colEvent.GetID()
End Function

Function ColEventGetMass1:TMass( colEvent:TColEvent )
	Return colEvent.GetMass1()
End Function

Function ColEventGetMass2:TMass( colEvent:TColEvent )
	Return colEvent.GetMass2()
End Function

Function ColEventGetZone:TZone( colEvent:TColEvent )
	Return colEvent.GetZone()
End Function

Function ColEventGetX:Float( colEvent:TColEvent )
	Return colEvent.GetX()
End Function

Function ColEventGetY:Float( colEvent:TColEvent )
	Return colEvent.GetY()
End Function

Function ColEventGetForce:Float( colEvent:TColEvent )
	Return colEvent.GetForce()
End Function

'Old procedural functions
Function Physics:TPhysics( width:Float, height:Float, gravY:Float, gravX:Float=0.0, i:Int=1 )
	Return TPhysics.Create(width,height,gravY,gravX,i)
End Function

Function UpdatePhysics()
	TPhysics.Update
End Function

Function EndPhysics()
	TPhysics.Free
End Function

Function SetPhysicsGravity( x:Float, y:Float )
	TPhysics.SetGravity x,y
End Function

Function SetPhysicsIterations( i:Int )
	TPhysics.SetIterations i
End Function

Function SetPhysicsBounds( originX:Float, originY:Float, width:Float, height:Float )
	TPhysics.SetBounds originX,originY,width,height
End Function

Function SetPhysicsFriction( frict:Float )
	TPhysics.SetFriction frict
End Function

Function UseShiftedGrid( value:Int )
	TPhysics.UseShiftedGrid value
End Function

Function SetGridSize( value:Int )
	TPhysics.SetGridSize value
End Function

